@@ -3,9 +3,9 @@ Application Version.
 
 Created on 12 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
-__version__ = "1.5.6"
+__version__ = "1.5.19"
