@@ -3,30 +3,21 @@ chart_frame.py
 
 Chart frame class for PyGPSClient application.
 
-This emulates a 4-channel "oscilloscope", allowing the user to plot
-up to 4 named data attributes over time. X-axis and Y-axis scale and
-ranges are all configurable.
-
-Plot limited only by the number of data points that can be comfortably
-held in memory.
+This emulates a multi-channel plotter, allowing the user to plot
+multiple named numeric data attributes from any parsed GNSS source over
+time. X-axis and Y-axis scale and ranges are all configurable.
 
 Created on 24 Nov 2024
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
 from random import choice
 from time import time
 from tkinter import (
-    ALL,
-    NE,
     NORMAL,
-    NW,
-    SE,
-    SW,
-    Canvas,
     E,
     Entry,
     Frame,
@@ -35,43 +26,41 @@ from tkinter import (
     S,
     Spinbox,
     StringVar,
+    TclError,
     W,
+    font,
 )
 
+from pygpsclient.canvas_plot import (
+    TAG_DATA,
+    TAG_GRID,
+    TAG_XLABEL,
+    TAG_YLABEL,
+    CanvasGraph,
+)
 from pygpsclient.globals import (
-    AXISCOL,
     BGCOL,
-    GRIDCOL,
+    ERRCOL,
+    FGCOL,
+    PLOTCOLS,
     READONLY,
-    RPTDELAY,
+    TRACEMODE_WRITE,
+    VALFLOAT,
+    VALNONSPACE,
     WIDGETU6,
-    AreaXY,
 )
-from pygpsclient.helpers import data2xy, fontheight, get_grid, scale_font, time2str
+from pygpsclient.helpers import time2str
 
-MAXCHANS = 4
-RESFONT = 28  # font size relative to widget size
-MINFONT = 8  # minimum font size
-PLOTWID = 1
-PLOTCOLS = ("yellow", "cyan", "magenta", "deepskyblue")
-GRIDMINCOL = "grey30"
-LBLGRID = 5
-GRIDSTEPS = get_grid(21)
-XLBLSTEPS = get_grid(LBLGRID)
-YLBLSTEPS = get_grid(LBLGRID)
-ERRCOL = "coral"
+OL_WID = 1
 LBLCOL = "white"
 CONTRASTCOL = "black"
-MODEINIT = "chart"
-AXISTAG = "axt"
-AXISLBLTAG = "axl"
 # total capacity depends on available free memory...
 TIMRANGE = [int(i * 10**n) for n in (1, 2, 3, 4) for i in (1, 2.4, 3.6, 4.8, 6)]
 DPTRANGE = [int(i * 10**n) for n in (3, 4, 5, 6) for i in (1, 2, 5)]
-XRANGE = 120
 CHARTMINY = 0
 CHARTMAXY = 100
 CHARTSCALE = 1
+FONTSCALE = 20
 MINY = "MinY {}"
 MAXY = "MaxY {}"
 
@@ -125,22 +114,19 @@ class ChartviewFrame(Frame):
 
         Frame.__init__(self, self.__master, *args, **kwargs)
 
-        self.chartsettings = self.__app.saved_config.get("chartsettings_d", {})
+        self.chartsettings = self.__app.configuration.get("chartsettings_d")
         def_w, def_h = WIDGETU6
         self.width = kwargs.get("width", def_w)
         self.height = kwargs.get("height", def_h)
         self.configure(bg=BGCOL)
-        self._font = None
-        self._fonth = 6
         self._xoff = 20  # chart X offset for labels
         self._yoff = 20  # chart Y offset for labels
         self._chart_data = {}
-        self._num_chans = self.chartsettings.get("numchn_n", MAXCHANS)
+        self._num_chans = self.chartsettings.get("numchn_n")
         if self._num_chans % 2:  # no channels must be even
             self._num_chans += 1
         self._plotcols = PLOTCOLS
         self._font = self.__app.font_sm
-        self._fonth = fontheight(self._font)
         # generate random plot colours for channels > 4
         if self._num_chans > 4:
             self._plotcols += tuple(
@@ -152,8 +138,9 @@ class ChartviewFrame(Frame):
         self._data_scale = [None] * self._num_chans
         self._data_miny = [None] * self._num_chans
         self._data_maxy = [None] * self._num_chans
-        self._mintim = 1e20
+        self._mintim = 0
         self._maxtim = 0
+        self._redraw = True
         self._timrange = StringVar()
         self._maxpoints = StringVar()
         for chn in range(self._num_chans):
@@ -164,9 +151,8 @@ class ChartviewFrame(Frame):
             self._data_maxy[chn] = StringVar()
         self._body()
         self._do_layout()
-        # self._set_fontsize()
-        self._attach_events()
         self.reset()
+        self._attach_events()
 
     def _body(self):
         """
@@ -180,8 +166,8 @@ class ChartviewFrame(Frame):
         self.grid_rowconfigure(0, weight=1)
         for i in range(1, 2 + self._num_chans):
             self.grid_rowconfigure(i, weight=0)
-        self._can_chartview = Canvas(
-            self, width=self.width, height=self.height, bg=BGCOL
+        self._canvas = CanvasGraph(
+            self.__app, self, width=self.width, height=self.height, bg=BGCOL
         )
         self._lbl_id = Label(
             self,
@@ -221,7 +207,7 @@ class ChartviewFrame(Frame):
         )
         self._lbl_maxpoints = Label(
             self,
-            text="Max Points/Chan",
+            text="Max Points",
             fg=LBLCOL,
             bg=BGCOL,
         )
@@ -255,11 +241,10 @@ class ChartviewFrame(Frame):
                 wrap=True,
                 textvariable=self._data_scale[chn],
                 state=NORMAL,
-                repeatdelay=RPTDELAY,
-                repeatinterval=RPTDELAY,
                 width=10,
                 fg=self._plotcols[chn],
                 bg=BGCOL,
+                buttonbackground=BGCOL,
             )
             self._spn_miny[chn] = Spinbox(
                 self,
@@ -267,11 +252,10 @@ class ChartviewFrame(Frame):
                 wrap=True,
                 textvariable=self._data_miny[chn],
                 state=NORMAL,
-                repeatdelay=RPTDELAY,
-                repeatinterval=RPTDELAY,
                 width=10,
                 fg=self._plotcols[chn],
                 bg=BGCOL,
+                buttonbackground=BGCOL,
             )
             self._spn_maxy[chn] = Spinbox(
                 self,
@@ -279,11 +263,10 @@ class ChartviewFrame(Frame):
                 wrap=True,
                 textvariable=self._data_maxy[chn],
                 state=NORMAL,
-                repeatdelay=RPTDELAY,
-                repeatinterval=RPTDELAY,
                 width=10,
                 fg=self._plotcols[chn],
                 bg=BGCOL,
+                buttonbackground=BGCOL,
             )
 
         self._spn_timrange = Spinbox(
@@ -292,11 +275,10 @@ class ChartviewFrame(Frame):
             wrap=True,
             textvariable=self._timrange,
             state=READONLY,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
             width=8,
             fg=LBLCOL,
             readonlybackground=BGCOL,
+            buttonbackground=BGCOL,
         )
         self._spn_maxpoints = Spinbox(
             self,
@@ -304,11 +286,10 @@ class ChartviewFrame(Frame):
             wrap=True,
             textvariable=self._maxpoints,
             state=READONLY,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
             width=8,
             fg=LBLCOL,
             readonlybackground=BGCOL,
+            buttonbackground=BGCOL,
         )
 
     def _do_layout(self):
@@ -316,7 +297,7 @@ class ChartviewFrame(Frame):
         Position widgets in frame.
         """
 
-        self._can_chartview.grid(column=0, row=0, columnspan=6, sticky=(N, S, E, W))
+        self._canvas.grid(column=0, row=0, columnspan=6, sticky=(N, S, E, W))
         self._lbl_id.grid(column=0, row=1, sticky=(W, E))
         self._lbl_name.grid(column=1, row=1, sticky=(W, E))
         self._lbl_scale.grid(column=2, row=1, sticky=(W, E))
@@ -339,17 +320,39 @@ class ChartviewFrame(Frame):
         """
 
         self.bind("<Configure>", self._on_resize)
-        self._can_chartview.bind("<Double-Button-1>", self._on_clear)
-        self._can_chartview.bind("<Double-Button-2>", self._on_clipboard)
-        self._can_chartview.bind("<Double-Button-3>", self._on_clipboard)
-        self._timrange.trace_add("write", self._on_save_settings)
-        self._maxpoints.trace_add("write", self._on_save_settings)
+        self._canvas.bind("<Double-Button-1>", self._on_clear)
+        self._canvas.bind("<Double-Button-2>", self._on_clipboard)
+        self._canvas.bind("<Double-Button-3>", self._on_clipboard)
+        self._timrange.trace_add(TRACEMODE_WRITE, self._on_update_config)
+        self._maxpoints.trace_add(TRACEMODE_WRITE, self._on_update_config)
         for chn in range(self._num_chans):
-            self._data_id[chn].trace_add("write", self._on_save_settings)
-            self._data_name[chn].trace_add("write", self._on_save_settings)
-            self._data_scale[chn].trace_add("write", self._on_save_settings)
-            self._data_miny[chn].trace_add("write", self._on_save_settings)
-            self._data_maxy[chn].trace_add("write", self._on_save_settings)
+            self._data_id[chn].trace_add(TRACEMODE_WRITE, self._on_update_config)
+            self._data_name[chn].trace_add(TRACEMODE_WRITE, self._on_update_config)
+            self._data_scale[chn].trace_add(TRACEMODE_WRITE, self._on_update_config)
+            self._data_miny[chn].trace_add(TRACEMODE_WRITE, self._on_update_config)
+            self._data_maxy[chn].trace_add(TRACEMODE_WRITE, self._on_update_config)
+
+    def _on_update_config(self, var, index, mode):  # pylint: disable=unused-argument
+        """
+        Update in-memory configuration if setting is changed.
+        """
+
+        try:
+            cst = {}
+            cst["numchn_n"] = int(self._num_chans)
+            cst["timrng_n"] = int(self._timrange.get())
+            cst["maxpoints_n"] = int(self._maxpoints.get())
+            for chn in range(self._num_chans):
+                cst[chn] = {}
+                cst[chn]["id_s"] = self._data_id[chn].get()
+                cst[chn]["name_s"] = self._data_name[chn].get()
+                cst[chn]["scale_f"] = float(self._data_scale[chn].get())
+                cst[chn]["miny_f"] = float(self._data_miny[chn].get())
+                cst[chn]["maxy_f"] = float(self._data_maxy[chn].get())
+            self.__app.configuration.set("chartsettings_d", cst)
+        except (ValueError, TclError):
+            pass
+        self._redraw = True
 
     def reset(self):
         """
@@ -373,11 +376,8 @@ class ChartviewFrame(Frame):
         Clear data.
         """
 
-        for chn in range(self._num_chans):
-            self._chart_data[chn] = []
-        self._mintim = 1e20
-        self._maxtim = 0
-        self._can_chartview.delete(ALL)
+        self._chart_data = {}
+        self._redraw = True
         self.update_frame()
 
     def _valid_settings(self) -> bool:
@@ -388,38 +388,14 @@ class ChartviewFrame(Frame):
         :rtype: bool
         """
 
-        try:
-            for chn in range(self._num_chans):
-                _ = float(self._data_scale[chn].get())
-                _ = float(self._data_miny[chn].get())
-                _ = float(self._data_maxy[chn].get())
-            _ = float(self._timrange.get())
-            return True
-        except ValueError:
-            self._disp_alert("Error: Invalid settings")
-            return False
-
-    def init_frame(self):
-        """
-        Initialise spectrum chart.
-        """
-
-        w, h = self.width, self.height
-        self._xoff = self._fonth * self._num_chans / 2 + 3  # chart X offset for labels
-        self._yoff = self._fonth + 3  # chart Y offset for labels
-        self._can_chartview.delete(ALL)
-
-        # draw grid
-        for i, p in enumerate(GRIDSTEPS):
-            y = (h - self._yoff) * p
-            col = AXISCOL if p in (0, 1.0) else GRIDMINCOL if i % LBLGRID else GRIDCOL
-            self._can_chartview.create_line(
-                self._xoff, y, w - self._xoff, y, fill=col, tags=AXISTAG
-            )
-            x = self._xoff + (w - self._xoff * 2) * p
-            self._can_chartview.create_line(
-                x, 0, x, h - self._yoff, fill=col, tags=AXISTAG
-            )
+        valid = True
+        for chn in range(self._num_chans):
+            valid &= self._ent_id[chn].validate(VALNONSPACE)
+            valid &= self._ent_name[chn].validate(VALNONSPACE)
+            valid &= self._spn_scale[chn].validate(VALFLOAT)
+            valid &= self._spn_miny[chn].validate(VALFLOAT)
+            valid &= self._spn_maxy[chn].validate(VALFLOAT)
+        return valid
 
     def update_data(self, parsed_data: object):
         """
@@ -433,6 +409,9 @@ class ChartviewFrame(Frame):
         except ValueError:
             maxpoints = DPTRANGE[2]  # 5000
 
+        now = round(time(), 0)  # time to nearest second
+        if now not in self._chart_data:
+            self._chart_data[now] = {}
         for chn in range(self._num_chans):
             mid = self._data_id[chn].get()
             name = self._data_name[chn].get()
@@ -470,20 +449,19 @@ class ChartviewFrame(Frame):
                 else:
                     continue
 
-            now = time()
-            self._chart_data[chn].append((now, val))
+            self._chart_data[now][chn] = val
 
             # update X axis (time) range
             self._mintim = min(now, self._mintim)
             self._maxtim = max(now, self._maxtim)
 
-            # limit number of data points per channel
-            while len(self._chart_data[chn]) > maxpoints:
-                self._chart_data[chn].pop(0)
-
             # flag if scaled value is out of range
             self.flag_outofrange(chn, val)
 
+        # limit number of data points
+        while len(self._chart_data) > maxpoints:
+            self._chart_data.pop(min(self._chart_data))
+
     def flag_outofrange(self, chn: int, val: float):
         """
         Flag if scaled value is over or under range.
@@ -519,6 +497,52 @@ class ChartviewFrame(Frame):
 
         self._update_plot(self._chart_data)
 
+    def init_frame(self):
+        """
+        Initialise spectrum chart.
+        """
+
+        # pylint: disable=consider-using-generator
+
+        # need tuples here to pass validation in create_graph()
+        yminval = tuple(
+            [float(self._data_miny[chn].get()) for chn in range(self._num_chans)]
+        )
+        ymaxval = tuple(
+            [float(self._data_maxy[chn].get()) for chn in range(self._num_chans)]
+        )
+        yleg = tuple(["" for chn in range(self._num_chans)])
+        dp = tuple([0 for chn in range(self._num_chans)])
+        ycol = self._plotcols
+
+        # Initialise graph
+        # only redraw the tags that have changed
+        tags = (TAG_GRID, TAG_XLABEL, TAG_YLABEL) if self._redraw else (TAG_XLABEL,)
+        self._maxtim = time()
+        self._mintim = self._maxtim - int(self._timrange.get())
+        self._canvas.create_graph(
+            xdatamax=self._maxtim,
+            xdatamin=self._mintim,
+            ydatamax=ymaxval,
+            ydatamin=yminval,
+            xtickmaj=5,
+            xtickmin=20,
+            ytickmaj=5,
+            ytickmin=20,
+            xdp=0,
+            ydp=dp,
+            xlegend="time",
+            xtimeformat="%H:%M:%S",
+            xcol=FGCOL,
+            ylegend=yleg,
+            ycol=ycol,
+            xlabels=True,
+            ylabels=True,
+            fontscale=FONTSCALE,
+            tags=tags,
+        )
+        self._redraw = False
+
     def _update_plot(self, data: dict):
         """
         Update chart plot with data.
@@ -527,199 +551,46 @@ class ChartviewFrame(Frame):
         :param int xrange: number of points
         """
 
+        # pylint: disable=no-member
+
         if not self._valid_settings():
             return
 
-        mintim = 1e20
-        maxtim = 0
-        w, h = self.width, self.height
         self.init_frame()
 
-        # set default ranges for all channels
-        minval = [CHARTMINY] * self._num_chans
-        maxval = [CHARTMAXY] * self._num_chans
-        scale = [CHARTSCALE] * self._num_chans
-
-        # get X axis (time) range for all channels and draw labels
-        mintim, maxtim = self._mintim, self._maxtim
-        mintim = maxtim - float(self._timrange.get())
-        bounds = AreaXY(mintim, CHARTMINY, maxtim, CHARTMAXY)
-        self._can_chartview.delete(AXISLBLTAG)
-        self._draw_xaxis_labels(w, h, bounds, mintim, maxtim)
         self._spn_timrange.configure(fg=LBLCOL, readonlybackground=BGCOL)
 
         # plot each channel's data points
         for chn in range(self._num_chans):
+            tm2 = self._mintim
+            vl2 = 0
+            for n, (tim, channels) in enumerate(data.items()):
 
-            chncol = self._plotcols[chn]
-            minval[chn] = float(self._data_miny[chn].get())
-            maxval[chn] = float(self._data_maxy[chn].get())
-            scale[chn] = float(self._data_scale[chn].get())
-            bounds = AreaXY(mintim, minval[chn], maxtim, maxval[chn])
+                try:
+                    val = channels[chn]
+                except KeyError:
+                    val = None
 
-            # draw Y axis (data value) labels for this channel
-            self._draw_yaxis_labels(w, h, bounds, minval[chn], maxval[chn], chn)
-
-            # plot each data point in channel
-            inr = False
-            for tim, val in data[chn]:
-
-                if val is None:  # not numeric
+                if val is None:
                     continue
 
-                if scale[chn] != 1:
-                    val /= scale[chn]  # scale data
+                scale = float(self._data_scale[chn].get())
+                if scale != 1:
+                    val /= scale  # scale data
 
-                # convert datapoint to canvas x,y coordinates
-                x, y = data2xy(
-                    w - self._xoff * 2,
-                    h - self._yoff,
-                    bounds,
-                    tim,
-                    val,
-                    self._xoff,
-                )
-                if x <= self._xoff:
-                    inr = False
-                # plot line
-                if inr:
-                    x2, y2 = x, y
-                    self._can_chartview.create_line(
-                        x1,
-                        y1,
-                        x2,
-                        y2,
-                        fill=chncol,
-                        width=PLOTWID,
-                        tags=f"plot_{chn:1d}",
+                tm1, vl1 = tm2, vl2
+                tm2, vl2 = tim, val
+                if n and tm1 > self._mintim:
+                    self._canvas.create_gline(
+                        tm1,
+                        vl1,
+                        tm2,
+                        vl2,
+                        fill=self._canvas.ycol[chn],
+                        width=OL_WID,
+                        chn=chn,
+                        tags=(TAG_DATA,),
                     )
-                    x1, y1 = x2, y2
-                else:
-                    x1, y1 = max(x, self._xoff), y
-                inr = True
-
-    def _draw_xaxis_labels(
-        self, w: int, h: int, bounds: AreaXY, mintim: float, maxtim: float
-    ):
-        """
-        Draw X axis (time) labels.
-
-        :param int w: canvas width
-        :param int h: canvas height
-        :param AreaXY bounds: data bounds
-        :param float mintim: minimum time
-        :param float maxtim: maximum time
-        """
-        # pylint: disable=too-many-arguments, too-many-positional-arguments
-
-        for g in XLBLSTEPS:
-            xval = mintim + (maxtim - mintim) * g
-            x, _ = data2xy(w - self._xoff * 2, h - self._yoff, bounds, xval, 0)
-            if g == 0:
-                anc = NW
-            elif g == 1:
-                anc = NE
-            else:
-                anc = N
-            self._can_chartview.create_text(
-                x + self._xoff,
-                h - self._yoff,
-                text=time2str(xval),
-                anchor=anc,
-                fill=AXISCOL,
-                font=self._font,
-                tags=AXISLBLTAG,
-            )
-
-    def _draw_yaxis_labels(
-        self,
-        w: int,
-        h: int,
-        bounds: AreaXY,
-        minval: float,
-        maxval: float,
-        chn: int,
-    ):
-        """
-        Draw Y axis (data value) labels for this channel.
-
-        :param int w: canvas width
-        :param int h: canvas height
-        :param AreaXY bounds: data bounds
-        :param float minval: minimum val for chn
-        :param float maxval: maximum val for chn
-        :param int chn: channel
-        """
-        # pylint: disable=too-many-arguments, too-many-positional-arguments
-
-        col = self._plotcols[chn]
-        yo = 2  # avoid edges
-        # y axis labels alternate left and right
-        if chn % 2:  # odd channels
-            x = w - yo - self._fonth * ((chn - 1) / 2)
-        else:  # even channels
-            x = yo + self._fonth * (chn / 2)
-
-        for g in YLBLSTEPS:
-            yval = minval + (maxval - minval) * g
-            _, y = data2xy(
-                w - self._xoff * MAXCHANS / 2, h - self._yoff, bounds, 0, yval
-            )
-            if g == 0:
-                anc = SW if chn % 2 else NW
-            elif g == 1:
-                y += yo * 2  # avoid edges
-                anc = SE if chn % 2 else NE
-            else:
-                anc = S if chn % 2 else N
-            self._can_chartview.create_text(
-                x,
-                y,
-                text=yval,
-                fill=col,
-                font=self._font,
-                angle=90,
-                anchor=anc,
-                tags=AXISLBLTAG,
-            )
-
-    def _disp_alert(self, msg):
-        """
-        Display alert message on canvas.
-
-        :param str msg: error message
-        """
-
-        w, h = self.width, self.height
-
-        self._can_chartview.delete(ALL)
-        self._can_chartview.create_text(
-            w / 2,
-            h / 2,
-            text=msg,
-            fill="orange",
-            font=self._font,
-            anchor=S,
-        )
-
-    def _on_save_settings(self, var, index, mode):  # pylint: disable=unused-argument
-        """
-        Save current settings to saved app config dict.
-        """
-
-        cst = {}
-        cst["numchn_n"] = self._num_chans
-        cst["timrng_n"] = self._timrange.get()
-        cst["maxpoints_n"] = self._maxpoints.get()
-        for chn in range(self._num_chans):
-            cst[chn] = {}
-            cst[chn]["id_s"] = self._data_id[chn].get()
-            cst[chn]["name_s"] = self._data_name[chn].get()
-            cst[chn]["scale_f"] = self._data_scale[chn].get()
-            cst[chn]["miny_f"] = self._data_miny[chn].get()
-            cst[chn]["maxy_f"] = self._data_maxy[chn].get()
-
-        self.__app.saved_config["chartsettings_d"] = cst
 
     def _on_clipboard(self, event):  # pylint: disable=unused-argument
         """
@@ -729,16 +600,23 @@ class ChartviewFrame(Frame):
         """
 
         csv = (
-            f"PyGPSClient Chart Data,{time2str(time(),'%Y-%M-%d-%H:%M:%S')},"
+            f"PyGPSClient Chart Data,{time2str(time(),'%Y-%m-%d-%H:%M:%S')},"
             f"Channels,{self._num_chans}\n"
         )
-        for chn in range(self._num_chans):
-            csv += (
-                f"{chn},{self._ent_id[chn].get()},{self._ent_name[chn].get()},"
-                f"{len(self._chart_data[chn])}\nTime,Value\n"
-            )
-            for tim, val in self._chart_data[chn]:
-                csv += f"{time2str(tim,'%H:%M:%S.%f')},{val}\n"
+        hdr = True
+        for tim, data in self._chart_data.items():
+            if hdr:
+                csv += "Timestamp"
+                for chn in range(self._num_chans):
+                    csv += f",{self._ent_id[chn].get()}.{self._ent_name[chn].get()}"
+                csv += "\n"
+                hdr = False
+            csv += f"{time2str(tim,'%Y-%m-%d-%H:%M:%S')}"
+            for chn in range(self._num_chans):
+                try:
+                    csv += f",{data[chn]}"
+                except KeyError:
+                    csv += ","
             csv += "\n"
         self.__master.clipboard_clear()
         self.__master.clipboard_append(csv)
@@ -752,7 +630,8 @@ class ChartviewFrame(Frame):
         """
 
         self.width, self.height = self.get_size()
-        self._font, self._fonth = scale_font(self.width, 6, 35, 16)
+        self._font = font.Font(size=int(min(self.width, self.height) / FONTSCALE))
+        self._redraw = True
 
     def get_size(self):
         """
@@ -763,4 +642,4 @@ class ChartviewFrame(Frame):
         """
 
         self.update_idletasks()  # Make sure we know about any resizing
-        return self._can_chartview.winfo_width(), self._can_chartview.winfo_height()
+        return self._canvas.winfo_width(), self._canvas.winfo_height()
