@@ -12,8 +12,8 @@ i.e. the first line
 
 Created on 12 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -32,7 +32,6 @@ from pygpsclient.globals import (
     FORMAT_BOTH,
     FORMAT_HEXSTR,
     FORMAT_HEXTAB,
-    FORMAT_PARSED,
     WIDGETU3,
 )
 from pygpsclient.strings import HALTTAGWARN
@@ -63,9 +62,7 @@ class ConsoleFrame(Frame):
         def_w, def_h = WIDGETU3
         self.width = kwargs.get("width", def_w)
         self.height = kwargs.get("height", def_h)
-        self._colortags = self.__app.frm_settings.config.get(
-            "colortags_l", self.__app.frm_settings.config.get("colortags", [])
-        )
+        self._colortags = self.__app.configuration.get("colortags_l")
         self._body()
         self._do_layout()
         self._attach_events()
@@ -98,7 +95,7 @@ class ConsoleFrame(Frame):
         self.sblogv.config(command=self.txt_console.yview)
 
         # making the textbox read only and fixed width font
-        self.txt_console.configure(font=FONT_FIXED, state="disabled")
+        self.txt_console.configure(state="disabled")
 
         # set up color tagging
         for match, color in self._colortags:
@@ -127,10 +124,9 @@ class ConsoleFrame(Frame):
         self.txt_console.bind("<Double-Button-3>", self._on_clipboard)
         # self.txt_console.tag_bind(HALT, "<1>", self._on_halt) # doesn't seem to work on MacOS
 
-    def update_console(self, consoledata: list):
+    def update_frame(self, consoledata: list):
         """
-        Print the latest data stream to the console in raw (NMEA) or
-        parsed (key,value pair) format.
+        Print the formatted data stream to the console.
 
         'maxlines' defines the maximum number of scrollable lines that are
         retained in the text box on a FIFO basis.
@@ -139,47 +135,45 @@ class ConsoleFrame(Frame):
             accumulated since last console update
         """
 
-        con = self.txt_console
-        consoleformat = self.__app.frm_settings.config.get(
-            "consoleformat_s", FORMAT_PARSED
-        )
-        colortagging = self.__app.frm_settings.config.get("colortag_b", 0)
-        maxlines = self.__app.frm_settings.config.get("maxlines_n", 100)
-        autoscroll = self.__app.frm_settings.config.get("autoscroll_b", 1)
-        self._halt = ""
-        con.configure(font=FONT_FIXED)
+        if not self.__app.configuration.get("autoscroll_b"):
+            return
 
+        consoleformat = self.__app.configuration.get("consoleformat_s")
+        colortagging = self.__app.configuration.get("colortag_b")
+        maxlines = self.__app.configuration.get("maxlines_n")
+        self._halt = ""
         consolestr = ""
+        self.txt_console.configure(font=FONT_TEXT)
         for raw_data, parsed_data, marker in consoledata:
             if consoleformat == FORMAT_BINARY:
-                data = str(raw_data).strip("\n")
+                data = f"{marker}{raw_data}".strip("\n")
             elif consoleformat == FORMAT_HEXSTR:
-                data = str(raw_data.hex())
+                data = f"{marker}{raw_data.hex()}"
             elif consoleformat == FORMAT_HEXTAB:
+                self.txt_console.configure(font=FONT_FIXED)
                 data = hextable(raw_data)
             elif consoleformat == FORMAT_BOTH:
+                self.txt_console.configure(font=FONT_FIXED)
                 data = f"{marker}{parsed_data}\n{hextable(raw_data)}"
             else:
-                con.configure(font=FONT_TEXT)
                 data = f"{marker}{parsed_data}"
             consolestr += data + "\n"
 
         numlinesbefore = self.numlines
-        con.configure(state="normal")
-        con.insert(END, consolestr)
+        self.txt_console.configure(state="normal")
+        self.txt_console.insert(END, consolestr)
 
         if colortagging:
-            self._tag_line(con, numlinesbefore, self.numlines)
+            self._tag_line(self.txt_console, numlinesbefore, self.numlines)
             if self._halt != "":
                 self._on_halt(None)
 
         while self.numlines > maxlines:
-            con.delete("1.0", "2.0")  # delete top line
+            self.txt_console.delete("1.0", "2.0")  # delete top line
 
-        if autoscroll:
-            con.see("end")
-        con.configure(state="disabled")
-        self.update_idletasks()
+        self.txt_console.see("end")
+        self.txt_console.configure(state="disabled")
+        self.txt_console.update_idletasks()
 
     def _tag_line(self, con, startline: int, endline: int):
         """
@@ -222,7 +216,7 @@ class ConsoleFrame(Frame):
         :param event event: HALT event
         """
 
-        self.__app.stream_handler.stop_read_thread()
+        self.__app.stream_handler.stop()
         self.__app.set_status(HALTTAGWARN.format(self._halt), ERRCOL)
         self.__app.conn_status = DISCONNECTED
 
