@@ -11,12 +11,13 @@ NB: this mechanism is dependent on receiving timely POLL responses.
 
 Created on 17 Aug 2022
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
 import logging
+import traceback
 from tkinter import (
     ALL,
     END,
@@ -38,7 +39,12 @@ from tkinter import (
 )
 
 from PIL import Image, ImageTk
-from pynmeagps import NMEA_PAYLOADS_POLL_PROP, NMEA_PAYLOADS_SET_PROP, NMEAMessage
+from pynmeagps import (
+    NMEA_MSGIDS_PROP,
+    NMEA_PAYLOADS_POLL_PROP,
+    NMEA_PAYLOADS_SET_PROP,
+    NMEAMessage,
+)
 from pyubx2 import (
     POLL,
     SET,
@@ -70,7 +76,7 @@ from pygpsclient.helpers import stringvar2val
 from pygpsclient.strings import LBLCFGGENERIC, LBLCFGGENERICNMEA
 
 # dimensions of scrollable attribute window
-SCROLLX = 300
+SCROLLX = 370
 SCROLLY = 300
 NMEA = "NMEA"
 UBX = "UBX"
@@ -79,6 +85,7 @@ NAK = "ACK-NAK"
 
 # following CFG types excluded from selection...
 CFG_EXCLUDED = (
+    "LSC",  # Locosys - many variants not implemented
     "CFG-DAT-NUM",  # deprecated
     "CFG-GEOFENCE",  # 'variable by size' groups not yet implemented - use pyubx2
     "CFG-NMEAv0",  # deprecated
@@ -90,6 +97,26 @@ CFG_EXCLUDED = (
 # alternative POLL dictionary names for where POLL command
 # doesn't correspond to SET (fudge for Quectel)
 ALT_POLL_NAMES = {
+    "AIR050": "AIR051",
+    "AIR058": "AIR059",
+    "AIR062": "AIR063",
+    "AIR066": "AIR067",
+    "AIR070": "AIR071",
+    "AIR072": "AIR073",
+    "AIR074": "AIR075",
+    "AIR080": "AIR081",
+    "AIR086": "AIR087",
+    "AIR100": "AIR101",
+    "AIR104": "AIR105",
+    "AIR400": "AIR401",
+    "AIR410": "AIR421",
+    "AIR420": "AIR411",
+    "AIR432": "AIR433",
+    "AIR434": "AIR435",
+    "AIR436": "AIR437",
+    "AIR864": "AIR865",
+    "AIR866": "AIR867",
+    "AIR6010": "AIR6011",
     "QTMCFGGEOFENCE_DIS": "QTMCFGGEOFENCE",
     "QTMCFGGEOFENCE_POLY": "QTMCFGGEOFENCE",
     "QTMCFGPPS_DIS": "QTMCFGPPS",
@@ -105,7 +132,9 @@ NMEA_POLL_VALS = {
     "msgname": "RMC",
     "msgver": 1,
     "porttype": 1,
+    "portType": 0,
     "portid": 1,
+    "portIndex": 0,
     "ppsindex": 1,
     "signalid": 1,
     "status": "R",
@@ -140,7 +169,7 @@ class Dynamic_Config_Frame(Frame):
         self.logger = logging.getLogger(__name__)
         self._protocol = kwargs.pop("protocol", "UBX")
 
-        Frame.__init__(self, self.__container.container, *args, **kwargs)
+        super().__init__(container.container, *args, **kwargs)
 
         self._img_send = ImageTk.PhotoImage(Image.open(ICON_SEND))
         self._img_pending = ImageTk.PhotoImage(Image.open(ICON_PENDING))
@@ -171,7 +200,7 @@ class Dynamic_Config_Frame(Frame):
             self,
             border=2,
             relief="sunken",
-            height=15,
+            height=10,
             justify=LEFT,
             exportselection=False,
         )
@@ -194,7 +223,7 @@ class Dynamic_Config_Frame(Frame):
             command=self._on_refresh,
             font=self.__app.font_md,
         )
-        self._lbl_command = Label(self, text="", anchor=W)
+        self._lbl_command = Label(self, text="", width=30, anchor=W)
         self._frm_container = Frame(self)
         self._can_container = Canvas(self._frm_container)
         self._frm_attrs = Frame(self._can_container)
@@ -258,8 +287,8 @@ class Dynamic_Config_Frame(Frame):
         self._lbx_cfg_cmd.delete(0, END)
         if self._protocol == NMEA:
             for i, cmd in enumerate(NMEA_PAYLOADS_SET_PROP):
-                if cmd[0:3] == "QTM" and cmd not in CFG_EXCLUDED:
-                    self._lbx_cfg_cmd.insert(i, cmd)
+                if cmd not in CFG_EXCLUDED:
+                    self._lbx_cfg_cmd.insert(i, f"P{cmd}")
         else:
             for i, cmd in enumerate(UBX_PAYLOADS_SET):
                 if cmd[0:3] == "CFG" and cmd not in CFG_EXCLUDED:
@@ -295,12 +324,16 @@ class Dynamic_Config_Frame(Frame):
 
         self._expected_response = None
         idx = self._lbx_cfg_cmd.curselection()
-        self._cfg_id = self._lbx_cfg_cmd.get(idx)
-        self._lbl_command.config(text=self._cfg_id)
+        self._cfg_id = self._lbx_cfg_cmd.get(idx)[1:]
         if self._protocol == NMEA:
+            cfgid = self._cfg_id.rsplit("_", 1)[0]
+            pde = NMEA_MSGIDS_PROP[cfgid].replace("Sets/Gets", "")
+            pdesc = f"P{cfgid} {pde}"
             pdic = NMEA_PAYLOADS_SET_PROP[self._cfg_id]
         else:  # UBX
+            pdesc = self._cfg_id
             pdic = UBX_PAYLOADS_SET[self._cfg_id]
+        self._lbl_command.config(text=f"{pdesc}")
         self._clear_widgets()
         self._add_widgets(pdic, 1, 0)
         self.update()
@@ -330,6 +363,8 @@ class Dynamic_Config_Frame(Frame):
                 # strip off any variant suffix from cfg_id
                 # e.g. "QTMCFGUART_CURR" -> "QTMCFGUART"
                 cfg_id = self._cfg_id.rsplit("_", 1)[0]
+                if cfg_id[0:3] == "UBX":  # strip id from msgId
+                    cfg_id = cfg_id[0:3]
                 msg = NMEAMessage("P", cfg_id, SET, **vals)
                 penddlg = NMEA_CFGOTHER
                 pendcfg = ("P" + cfg_id,)
@@ -341,16 +376,16 @@ class Dynamic_Config_Frame(Frame):
 
             # send message, update status and await response
             self.__container.send_command(msg)
-            # self.logger.debug(f"command {msg.serialize()}")
             self._lbl_send_command.config(image=self._img_pending)
             self.__container.set_status(
-                f"{self._cfg_id} SET message sent",
+                f"P{self._cfg_id} SET message sent",
             )
             for msgid in pendcfg:
                 self.__container.set_pending(msgid, penddlg)
             self._expected_response = SET
 
         except ValueError as err:
+            self.logger.debug(traceback.format_exc())
             self.__container.set_status(
                 f"INVALID! {nam}, {att}: {err}",
                 ERRCOL,
@@ -370,7 +405,7 @@ class Dynamic_Config_Frame(Frame):
             return
 
         msg = penddlg = pendcfg = None
-        # use alternate names for some NMEA PQTM POLL commands
+        # use alternate names for some NMEA PAIR/PQTM POLL commands
         cfg_id = ALT_POLL_NAMES.get(self._cfg_id, self._cfg_id)
         # set any POLL arguments to specified or default values
         # e.g. portid = "1", msgname="RMC"
@@ -390,15 +425,14 @@ class Dynamic_Config_Frame(Frame):
 
         if msg is not None:
             self.__container.send_command(msg)
-            # self.logger.debug(f"poll {msg.serialize()}")
-            self.__container.set_status(f"{cfg_id} POLL message sent", INFOCOL)
+            self.__container.set_status(f"P{cfg_id} POLL message sent", INFOCOL)
             self._lbl_send_command.config(image=self._img_pending)
             for msgid in pendcfg:
                 self.__container.set_pending(msgid, penddlg)
             self._expected_response = POLL
         else:  # CFG cannot be POLLed
             self.__container.set_status(
-                f"{cfg_id} No POLL available",
+                f"P{cfg_id} No POLL available",
             )
             self._lbl_send_command.config(image=self._img_unknown)
 
@@ -453,7 +487,6 @@ class Dynamic_Config_Frame(Frame):
         """
 
         ok = False
-        # self.logger.debug(f"incoming status msg: {msg}")
         # strip off any variant suffix from cfg_id
         # e.g. "QTMCFGUART_CURR" -> "PQTMCFGUART"
         cfg_id = (
@@ -476,10 +509,10 @@ class Dynamic_Config_Frame(Frame):
                     self._update_widgets(msg)
 
             if ok:
-                self.__container.set_status(f"{cfg_id} message acknowledged", OKCOL)
+                self.__container.set_status(f"P{cfg_id} message acknowledged", OKCOL)
                 self._lbl_send_command.config(image=self._img_confirmed)
             else:
-                self.__container.set_status(f"{cfg_id} message rejected", ERRCOL)
+                self.__container.set_status(f"P{cfg_id} message rejected", ERRCOL)
                 self._lbl_send_command.config(image=self._img_warn)
             self.update()
 
@@ -613,10 +646,12 @@ class Dynamic_Config_Frame(Frame):
         :param Entry wdg: Entry widget
         """
 
-        cfg = self._cfg_id.split("_")[0]
+        cfg = self._cfg_id  # .split("_")[0]
+        # use alternate names for some NMEA PAIR/PQTM POLL commands
+        cfg = ALT_POLL_NAMES.get(cfg, cfg)
         if (
             self._protocol == NMEA
-            and (cfg[:3] == "QTM" and nam != "status")  # ignore Quectel status
+            and (cfg[:3] in ("QTM", "AIR") and nam != "status")  # ignore Quectel status
             and nam in NMEA_PAYLOADS_POLL_PROP.get(cfg, {})
         ) or (self._protocol == UBX and nam in UBX_PAYLOADS_POLL.get(cfg, {})):
             wdg.configure(highlightbackground=INFOCOL, highlightcolor=INFOCOL)
