@@ -12,8 +12,8 @@ This handles all the file i/o, including:
 
 Created on 16 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -28,6 +28,7 @@ from pyubx2 import hextable
 from pygpsclient.globals import (
     APPNAME,
     CONFIGFILE,
+    ERRCOL,
     FORMAT_BINARY,
     FORMAT_BOTH,
     FORMAT_HEXSTR,
@@ -66,6 +67,7 @@ class FileHandler:
         self._logname = None
         self._logfile = None
         self._trackpath = None
+        self._databasepath = None
         self._trackname = None
         self._trackfile = None
         self._configpath = None
@@ -130,7 +132,7 @@ class FileHandler:
                 if err != "":
                     raise ValueError(err)
         except (ValueError, OSError, json.JSONDecodeError) as err:
-            return (None, None, str(err))
+            return (filename, None, str(err))
 
         return (filename, config, "")
 
@@ -218,17 +220,25 @@ class FileHandler:
             return None  # User cancelled
         return self._logpath
 
-    def open_logfile(self):
+    def open_logfile(self) -> int:
         """
         Open logfile.
+
+        :return: 0 = error, 1 = ok
+        :rtype: int
         """
 
         # pylint: disable=consider-using-with
 
-        self._logpath = self.__app.frm_settings.config.get("logpath_s", HOME)
-        self._lines = 0
-        _, self._logname = set_filename(self._logpath, "data", "log")
-        self._logfile = open(self._logname, "a+b")
+        try:
+            self._logpath = self.__app.configuration.get("logpath_s")
+            self._lines = 0
+            _, self._logname = set_filename(self._logpath, "data", "log")
+            self._logfile = open(self._logname, "a+b")
+            return 1
+        except FileNotFoundError as err:
+            self.__app.set_status(f"{err}", ERRCOL)
+            return 0
 
     def write_logfile(self, raw_data, parsed_data):
         """
@@ -238,10 +248,10 @@ class FileHandler:
         """
 
         if self._logfile is None:
-            self.open_logfile()
+            if not self.open_logfile():
+                return
 
-        settings = self.__app.frm_settings.config
-        lfm = settings["logformat_s"]
+        lfm = self.__app.configuration.get("logformat_s")
         data = []
         if lfm in (FORMAT_PARSED, FORMAT_BOTH):
             data.append(parsed_data)
@@ -293,15 +303,23 @@ class FileHandler:
             return None  # User cancelled
         return self._trackpath
 
-    def open_trackfile(self):
+    def open_trackfile(self) -> int:
         """
         Open track file and create GPX track header tags.
+
+        :return: 0 = error, 1 = ok
+        :rtype: int
         """
 
         # pylint: disable=consider-using-with
 
-        _, self._trackname = set_filename(self._trackpath, "track", "gpx")
-        self._trackfile = open(self._trackname, "a", encoding="utf-8")
+        try:
+            self._trackpath = self.__app.configuration.get("trackpath_s")
+            _, self._trackname = set_filename(self._trackpath, "track", "gpx")
+            self._trackfile = open(self._trackname, "a", encoding="utf-8")
+        except FileNotFoundError as err:
+            self.__app.set_status(f"{err}", ERRCOL)
+            return 0
 
         date = datetime.now().isoformat() + "Z"
         gpxtrack = (
@@ -319,6 +337,8 @@ class FileHandler:
         except ValueError:
             pass
 
+        return 1
+
     def add_trackpoint(self, lat: float, lon: float, **kwargs):
         """
         Creates GPX track point from provided parameters.
@@ -363,8 +383,9 @@ class FileHandler:
         trkpnt += "</trkpt>"
 
         try:
-            self._trackfile.write(trkpnt)
-            self._trackfile.flush()
+            if self._trackfile is not None:
+                self._trackfile.write(trkpnt)
+                self._trackfile.flush()
         except (IOError, ValueError):
             pass
 
@@ -444,3 +465,19 @@ class FileHandler:
                 )
 
             self._last_track_update = datetime.now()
+
+    def set_database_path(self, initdir=HOME) -> Path:
+        """
+        Set database directory.
+
+        :param str initdir: initial directory (HOME)
+        :return: file path
+        :rtype: str
+        """
+
+        self._databasepath = filedialog.askdirectory(
+            title=SAVETITLE, initialdir=initdir, mustexist=True
+        )
+        if self._databasepath in ((), ""):
+            return None  # User cancelled
+        return self._databasepath
