@@ -7,8 +7,8 @@ Collection of global constants
 
 Created on 14 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -22,16 +22,18 @@ from tkinter import Canvas
 
 from pyubx2 import GET, POLL, SET, SETPOLL
 
+# Type Declarations
 Point = namedtuple("Point", ["lat", "lon"])
 # Area convention is minlat, minlon, maxlat, maxlon
 Area = namedtuple("Area", ["lat1", "lon1", "lat2", "lon2"])
-PointXY = namedtuple("Point", ["x", "y"])
-AreaXY = namedtuple("Area", ["x1", "y1", "x2", "y2"])
+TrackPoint = namedtuple("TrackPoint", ["lat", "lon", "tim", "ele", "spd"])
+PointXY = namedtuple("PointXY", ["x", "y"])
+AreaXY = namedtuple("AreaXY", ["x1", "y1", "x2", "y2"])
 
 
-def create_circle(self, x, y, r, **kwargs):
+def create_circle(self: Canvas, x: int, y: int, r: int, **kwargs):
     """
-    Helper method to simplify drawing circles on canvas
+    Extends tkinter.Canvas class to simplify drawing compass grids on canvas.
 
     :param int x: x coordinate
     :param int y: y coordinate
@@ -43,10 +45,38 @@ def create_circle(self, x, y, r, **kwargs):
 
 Canvas.create_circle = create_circle
 
+# Default colors
+AXISCOL = "#E5E5E5"  # default plot axis color
+BGCOL = "#3D3D3D"  # default widget background color
+ERRCOL = "#FA8072"  # default error message color
+FGCOL = "#FFFFFF"  # default widget foreground color
+GNSS_LIST = {
+    0: ("GPS", "#6495ED"),
+    1: ("SBA", "#FF8000"),
+    2: ("GAL", "#008B00"),
+    3: ("BEI", "#9F79EE"),
+    4: ("IME", "#EE82EE"),
+    5: ("QZS", "#CDCD00"),
+    6: ("GLO", "#CD5C5C"),
+    7: ("NAV", "#D2B48C"),
+}
+GRIDLEGEND = "#B0B0B0"  # default grid legend color
+GRIDMAJCOL = "#666666"  # default grid major tick color
+GRIDMINCOL = "#4D4D4D"  # default grid minor tick color
+INFOCOL = "#5CACEE"  # default info message color
+OKCOL = "#008000"  # default OK message color
+PLOTCOLS = ("#FFFF00", "#00FFFF", "#FF00FF", "#00BFFF")
+PNTCOL = "#FF8000"  # default plot point color
+
+# Protocols to be used in protocol mask (others defined in gnss_reader.py)
+SPARTN_PROTOCOL = 32
+MQTT_PROTOCOL = 64
+TTY_PROTOCOL = 128
+
+# Various global constants
 HOME = Path.home()
 APPNAME = __name__.split(".", 1)[0]  # i.e. "pygpsclient"
-AXISCOL = "white"
-BGCOL = "gray24"  # default widget background color
+ASCII = "ascii"
 BPSRATES = (
     9600,
     19200,
@@ -59,6 +89,7 @@ BPSRATES = (
     1000000,
     4800,
 )
+BSR = "backslashreplace"
 CFG = "cfg"
 CLASS = "cls"
 COLORTAGS = "colortags"
@@ -82,7 +113,7 @@ DDD = "DD.D"
 DEFAULT_BUFSIZE = 4096
 DEFAULT_PASSWORD = "password"  # nosec
 DEFAULT_PORT = 50010
-DEFAULT_TLS_PORTS = (443, 2102)
+DEFAULT_TLS_PORTS = (443, 2102, 8443, 50443, 58443)
 DEFAULT_REGION = "eu"
 DEFAULT_SERVER = "localhost"
 DEFAULT_USER = "anon"
@@ -91,10 +122,8 @@ DISCONNECTED = 0
 DMM = "DM.M"
 DMS = "D.M.S"
 ECEF = "ECEF"
-ENABLE_CFG_OTHER = True  # enable CFG=* Other Configuration command panel
-ERRCOL = "salmon"  # default invalid data entry field background color
+ENABLE_CFG_LEGACY = True  # enable CFG=* Other Configuration command panel
 ERROR = "ERR!"
-FGCOL = "white"  # default widget foreground color
 FILEREAD_INTERVAL = 0.02  # delay between successive datalog file reads (seconds)
 FONT_FIXED = "TkFixedFont"
 FONT_MENU = "TkMenuFont"
@@ -115,20 +144,11 @@ FRAME = "frm"
 GGA_INTERVALS = ("None", "2", "5", "10", "60", "120")
 GITHUB_URL = "https://github.com/semuconsulting/PyGPSClient"
 GLONASS_NMEA = True  # use GLONASS NMEA SVID (65-96) rather than slot (1-24)
+GNSS = "GNSS"
 GNSS_EOF_EVENT = "<<gnss_eof>>"
 GNSS_ERR_EVENT = "<<gnss_error>>"
 GNSS_EVENT = "<<gnss_read>>"
 GNSS_TIMEOUT_EVENT = "<<gnss_timeout>>"
-GNSS_LIST = {
-    0: ("GPS", "cornflowerblue"),
-    1: ("SBA", "orange"),
-    2: ("GAL", "green4"),
-    3: ("BEI", "mediumpurple2"),
-    4: ("IME", "violet"),
-    5: ("QZS", "yellow"),
-    6: ("GLO", "indianred"),
-    7: ("NAV", "grey60"),
-}
 GPSEPOCH0 = datetime(1980, 1, 6)  # for Wno and Tow calculations
 GPX_NS = (
     'xmlns="http://www.topografix.com/GPX/1/1" '
@@ -138,8 +158,7 @@ GPX_NS = (
     'http://www.topografix.com/GPX/1/1/gpx.xsd"'
 )
 GPX_TRACK_INTERVAL = 1  # minimum GPS track update interval (seconds)
-GRIDCOL = "grey40"
-GUI_UPDATE_INTERVAL = 0.5  # minimum GUI widget update interval (seconds)
+GUI_UPDATE_INTERVAL = 0.5  # GUI widget update interval (seconds)
 ICON_APP128 = path.join(DIRNAME, "resources/app-128.png")
 ICON_BLANK = path.join(DIRNAME, "resources/blank-1-24.png")
 ICON_CONFIRMED = path.join(DIRNAME, "resources/iconmonstr-check-mark-8-24.png")
@@ -157,6 +176,7 @@ ICON_NMEACONFIG = path.join(DIRNAME, "resources/iconmonstr-gear-2-24-brown.png")
 ICON_NOCLIENT = path.join(DIRNAME, "resources/iconmonstr-noclient-10-24.png")
 ICON_NOTRANSMIT = path.join(DIRNAME, "resources/iconmonstr-notransmit-10-24.png")
 ICON_NTRIPCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-4-24.png")
+ICON_TTYCONFIG = path.join(DIRNAME, "resources/icon-tty-24-green.png")
 ICON_PENDING = path.join(DIRNAME, "resources/iconmonstr-time-6-24.png")
 ICON_PLAY = path.join(DIRNAME, "resources/iconmonstr-media-control-48-24.png")
 ICON_POS = path.join(DIRNAME, "resources/iconmonstr-plus-lined-24.png")
@@ -168,6 +188,7 @@ ICON_SEND = path.join(DIRNAME, "resources/iconmonstr-arrow-12-24.png")
 ICON_SERIAL = path.join(DIRNAME, "resources/usbport-1-24.png")
 ICON_SOCKET = path.join(DIRNAME, "resources/ethernet-1-24.png")
 ICON_SPARTNCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-3-24.png")
+# ICON_SPARTNDISABLE = path.join(DIRNAME, "resources/iconmonstr-antenna-3-24-greyed.png")
 ICON_START = path.join(DIRNAME, "resources/marker_start.png")
 ICON_STOP = path.join(DIRNAME, "resources/iconmonstr-stop-1-24.png")
 ICON_TRANSMIT = path.join(DIRNAME, "resources/iconmonstr-transmit-10-24.png")
@@ -176,46 +197,27 @@ ICON_UNDO = path.join(DIRNAME, "resources/iconmonstr-undo-24.png")
 ICON_UNKNOWN = path.join(DIRNAME, "resources/clear-1-24.png")
 ICON_WARNING = path.join(DIRNAME, "resources/iconmonstr-warning-1-24.png")
 IMG_WORLD = path.join(DIRNAME, "resources/world.png")
-ICON_SPONSOR = path.join(DIRNAME, "resources/bmc-full-logo-no-background.png")
-IMG_WORLD_CALIB = Area(-90, -180, 90, 180)
-INFOCOL = "steelblue2"
-KM2M = 1000
-KM2MIL = 0.621371
-KM2NMIL = 0.5399568
-KNOWNGPS = (
-    "CP210",
-    "FT230",
-    "FT232",
-    "garmin",
-    "gnss",
-    "gps",
-    "magellan",
-    "navstar",
-    "septentrio",
-    "sirf",
-    "trimble",
-    "u-blox",
-    "ublox",
-    "USB Serial",
-    "USB to UART",
-    "USB UART",
-    "USB_to_UART",
-    "USER-DEFINED",
-)
-KPH2KNT = 0.5399568
-KPH2MPH = 0.621371
-KPH2MPS = 0.2777776918389111005
+ICON_SPONSOR = path.join(DIRNAME, "resources/bmac-logo-60.png")
+IMG_WORLD_BOUNDS = Area(-90, -180, 90, 180)
+LBAND = "LBAND"
 LICENSE_URL = "https://github.com/semuconsulting/PyGPSClient/blob/master/LICENSE"
-M2FT = 3.28084
 MAPAPI_URL = "https://developer.mapquest.com/user/login/sign-up"
+MQTTIPMODE = 0
+MQTTLBANDMODE = 1
+MINHEIGHT = 600
+MINWIDTH = 800
 CUSTOM = "custom"
-MAP = "map"
-SAT = "sat"
+IMPORT = "import"
 WORLD = "world"
+LC29H = "Quectel LC29H"
+LG290P = "Quectel LG29P/LG580P"
 MAX_SNR = 60  # upper limit of graphview snr axis
 MAXLOGLINES = 10000  # maximum number of 'lines' per datalog file
+MIN_GUI_UPDATE_INTERVAL = 0.1  # minimum GUI widget update interval (seconds)
+MAXFLOAT = 2e20
+MINFLOAT = -MAXFLOAT
+MOSAIC_X5 = "Septentrio Mosaic X5"
 MQAPIKEY = "mqapikey"
-MQTT_PROTOCOL = 16
 MSGMODES = {
     "GET": GET,
     "SET": SET,
@@ -223,10 +225,9 @@ MSGMODES = {
     "SETPOLL": SETPOLL,
 }
 NOPORTS = 3
+NTRIP = "NTRIP"
 NTRIP_EVENT = "<<ntrip_read>>"
-OKCOL = "green"
-PNTCOL = "orange"
-POPUP_TRANSIENT = True  # whether pop-up config dialogs are always on top
+PASSTHRU = "Passthrough"
 PORTIDS = ("0 I2C", "1 UART1", "2 UART2", "3 USB", "4 SPI")
 PUBLICIP_URL = "https://ipinfo.io/json"
 PYPI_URL = "https://pypi.org/pypi/PyGPSClient"
@@ -234,10 +235,12 @@ QUITONERRORDEFAULT = 1
 RCVR_CONNECTION = "USB,UART1"  # default GNSS receiver connection port(s)
 ROMVER_NEW = "23.01"  # min device ROM version using configuration database
 READONLY = "readonly"
-RPTDELAY = 500  # repeat delay for spinboxes
+RESIZE = "resize"
+ROUTE = "route"
 RXMMSG = "RXM-SPARTN-KEY"
 SAT_EXPIRY = 10  # how long passed satellites are kept in the sky and graph view
-SAVED_CONFIG = "saved_config"
+
+SCREENSCALE = 0.8  # screen resolution scaling factor
 SOCK_NTRIP = "NTRIP CASTER"
 SOCK_SERVER = "SOCKET SERVER"
 SOCKCLIENT_HOST = "localhost"
@@ -257,11 +260,11 @@ SPARTN_BASEDATE_DATASTREAM = 0
 SPARTN_OUTPORT = 8883
 SPARTN_PPREGIONS = ("eu", "us", "jp", "kr", "au")
 SPARTN_PPSERVER_URL = "pp.services.u-blox.com"
-SPARTN_PROTOCOL = 8
 SPARTN_SOURCE_IP = 0
 SPARTN_SOURCE_LB = 1
 SPONSOR_URL = "https://buymeacoffee.com/semuconsulting"
 SQRT2 = 0.7071067811865476  # square root of 2
+STATUSPRIORITY = {INFOCOL: 0, "blue": 0, OKCOL: 2, "green": 1, ERRCOL: 3, "red": 3}
 THD = "thd"
 TIME0 = datetime(1970, 1, 1)  # basedate for time()
 TIMEOUTS = (
@@ -278,19 +281,52 @@ TIMEOUTS = (
 TOPIC_IP = "/pp/ip/{}"
 TOPIC_MGA = "/pp/ubx/mga"
 TOPIC_RXM = "/pp/ubx/0236/ip"
+TRACK = "track"
+TRACEMODE_WRITE = "write"
+TTYOK = ("OK", "$R:")
+TTYERR = ("ERROR", "$R?")
+TTYMARKER = "TTY<<"
 UBXPRESETS = "ubxpresets"
 UBXSIMULATOR = "ubxsimulator"
-UI = "Imperial mph"
-UIK = "Imperial knots"
-UMK = "Metric kmph"
-UMM = "Metric m/s"
 UTF8 = "utf-8"
+VALBLANK = 1
+VALNONBLANK = 2
+VALNONSPACE = 3
+VALINT = 4
+VALFLOAT = 5
+VALURL = 6
+VALHEX = 7
+VALDMY = 8
+VALLEN = 9
+VALBOOL = 10
+WAYPOINT = "waypoint"
 WIDGETU1 = (200, 200)  # small widget size
 WIDGETU2 = (300, 200)  # medium widget size
 WIDGETU3 = (800, 200)  # Console size
 WIDGETU4 = (500, 500)  # GPX Track viewer size
 WIDGETU6 = (400, 200)  # Chart size
 XML_HDR = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
+ZED_F9 = "u-blox ZED-F9"
+ZED_X20 = "u-blox ZED-X20"
+
+# Conversion factors
+UI = "Imperial mph"
+UIK = "Imperial knots"
+UMK = "Metric kmph"
+UMM = "Metric m/s"
+KM2M = 1000
+KM2MIL = 0.621371
+KM2NMIL = 0.5399568
+KPH2KNT = 0.5399568
+KPH2MPH = 0.6213712
+KPH2MPS = 0.2777778
+M2FT = 3.28084
+M2MIL = 0.0006213712
+M2NMIL = 0.0005399568
+M2KM = 0.001
+MPS2MPH = 2.236936
+MPS2KPH = 3.6
+MPS2KNT = 1.943844
 
 # UBX & NMEA config widget signifiers - used
 # to identify which widget should receive the
@@ -313,6 +349,32 @@ UBX_MONRF = 14
 NMEA_MONHW = 15
 NMEA_PRESET = 16
 NMEA_CFGOTHER = 17
+SERVERCONFIG = 18
+SBF_MONHW = 19
+
+# keep KNOWNGPS all lower case
+KNOWNGPS = (
+    "cp210",
+    "ft230",
+    "ft232",
+    "garmin",
+    "gnss",
+    "gps",
+    "iousbhostdevice",
+    "magellan",
+    "navstar",
+    "septentrio",
+    "sirf",
+    "trimble",
+    "u-blox",
+    "ublox",
+    "usb serial",
+    "usb to uart",
+    "usb uart",
+    "usb dual_serial",
+    "usb_to_uart",
+    "user-defined",
+)
 
 # map of fix values to descriptions
 # the keys in this map are a concatenation of NMEA/UBX
@@ -381,4 +443,14 @@ FIXLOOKUP = {
     "NAV2-STATUS5": "TIME ONLY",
     "NAV2-STATUS6": "RTK FLOAT",  # carrSoln
     "NAV2-STATUS7": "RTK FIXED",
+    "PVTGeodetic0": "NO FIX",
+    "PVTGeodetic1": "3D",
+    "PVTGeodetic2": "RTK",
+    "PVTGeodetic3": "3D",
+    "PVTGeodetic4": "RTK FIXED",
+    "PVTGeodetic5": "RTK FLOAT",
+    "PVTGeodetic6": "SBAS",
+    "PVTGeodetic7": "RTK FIXED",
+    "PVTGeodetic8": "RTK FLOAT",
+    "PVTGeodetic10": "PPP",
 }
