@@ -5,8 +5,8 @@ This is the pop-up dialog for the custom map import function.
 
 Created on 14 Sep 2024
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -14,26 +14,21 @@ Created on 14 Sep 2024
 
 from tkinter import (
     ALL,
-    BOTH,
     DISABLED,
     NORMAL,
-    NW,
-    YES,
     Button,
-    Canvas,
+    Checkbutton,
     E,
     Entry,
     Frame,
+    IntVar,
     Label,
     N,
     S,
     StringVar,
-    Toplevel,
     W,
 )
 
-from PIL import Image, ImageTk
-
 try:
     from rasterio import open as openraster  # pylint: disable=import-error
     from rasterio.warp import transform_bounds  # pylint: disable=import-error
@@ -42,17 +37,19 @@ try:
 except (ImportError, ModuleNotFoundError):
     HASRASTERIO = False
 
+from pygpsclient.canvas_map import CanvasMap
 from pygpsclient.globals import (
     BGCOL,
     ERRCOL,
-    HOME,
-    ICON_EXIT,
-    ICON_LOAD,
-    ICON_SEND,
+    IMPORT,
     INFOCOL,
-    POPUP_TRANSIENT,
+    TRACEMODE_WRITE,
+    VALFLOAT,
+    Area,
 )
+from pygpsclient.helpers import validate  # pylint: disable=unused-import
 from pygpsclient.strings import DLGTIMPORTMAP
+from pygpsclient.toplevel_dialog import ToplevelDialog
 
 # profile chart parameters:
 AXIS_XL = 35  # x axis left offset
@@ -62,9 +59,10 @@ ELEAX_COL = "green4"  # color of elevation plot axis
 ELE_COL = "palegreen3"  # color of elevation plot
 SPD_COL = INFOCOL  # color of speed plot
 MD_LINES = 2  # number of lines of metadata
+MINDIM = (456, 418)
 
 
-class ImportMapDialog(Toplevel):
+class ImportMapDialog(ToplevelDialog):
     """ImportMapDialog class."""
 
     def __init__(self, app, *args, **kwargs):
@@ -72,54 +70,53 @@ class ImportMapDialog(Toplevel):
 
         self.__app = app
         # self.__master = self.__app.appmaster  # link to root Tk window
-        Toplevel.__init__(self, app)
-        if POPUP_TRANSIENT:
-            self.transient(self.__app)
-        self.resizable(True, True)
-        self.title(DLGTIMPORTMAP)  # pylint: disable=E1102
-        self.protocol("WM_DELETE_WINDOW", self.on_exit)
-        self._img_load = ImageTk.PhotoImage(Image.open(ICON_LOAD))
-        self._img_send = ImageTk.PhotoImage(Image.open(ICON_SEND))
-        self._img_exit = ImageTk.PhotoImage(Image.open(ICON_EXIT))
+        super().__init__(app, DLGTIMPORTMAP, MINDIM)
         self.width = int(kwargs.get("width", 400))
         self.height = int(kwargs.get("height", 400))
-        self.mheight = int(self.height * 0.75)
+        self._first = IntVar()
         self._lonmin = StringVar()
         self._latmin = StringVar()
         self._lonmax = StringVar()
         self._latmax = StringVar()
-        self._mapimg = None
-        self._thmbimg = None
-        self._initdir = HOME
         self._custommap = ""
 
         self._body()
         self._do_layout()
         self._attach_events()
         self._reset()
+        self._finalise()
 
     def _body(self):
         """
         Create widgets.
         """
 
-        self._frm_map = Frame(self, borderwidth=2, relief="groove", bg=BGCOL)
-        self._frm_controls = Frame(self, borderwidth=2, relief="groove")
-        self._canvas_map = Canvas(
-            self._frm_map, width=self.width, height=self.mheight, bg=BGCOL
+        self._frm_body = Frame(self.container, borderwidth=2, relief="groove")
+        self._can_mapview = CanvasMap(
+            self.__app, self._frm_body, width=self.width, height=self.height, bg=BGCOL
         )
+        self._frm_controls = Frame(self._frm_body, borderwidth=2, relief="groove")
         self._btn_load = Button(
             self._frm_controls,
-            image=self._img_load,
+            image=self.img_load,
             width=40,
             command=self._on_load,
         )
         self._btn_import = Button(
             self._frm_controls,
-            image=self._img_send,
+            image=self.img_send,
             width=40,
             command=self._on_import,
         )
+        self._btn_redraw = Button(
+            self._frm_controls,
+            image=self.img_redraw,
+            width=40,
+            command=self._on_redraw,
+        )
+        self._chk_first = Checkbutton(
+            self._frm_controls, text="First?", variable=self._first
+        )
         self._lbl_min = Label(self._frm_controls, text="Minimum:")
         self._lbl_minlat = Label(self._frm_controls, text="Latitude")
         self._ent_minlat = Entry(
@@ -138,26 +135,18 @@ class ImportMapDialog(Toplevel):
         self._ent_maxlon = Entry(
             self._frm_controls, width=10, textvariable=self._lonmax
         )
-        self._btn_exit = Button(
-            self._frm_controls,
-            image=self._img_exit,
-            width=40,
-            command=self.on_exit,
-        )
-        self._lbl_status = Label(self._frm_controls, text="", anchor=W)
 
     def _do_layout(self):
         """
         Arrange widgets.
         """
-
-        self._frm_map.grid(column=0, row=0, sticky=(N, S, E, W))
+        self._frm_body.grid(column=0, row=0, sticky=(N, S, E, W))
+        self._can_mapview.grid(column=0, row=0, sticky=(N, S, E, W))
         self._frm_controls.grid(column=0, row=1, sticky=(W, E))
-        self._canvas_map.pack(fill=BOTH, expand=YES)
         self._btn_load.grid(column=0, row=0, padx=3, pady=3)
-        self._btn_import.grid(column=1, row=0, padx=3, pady=3)
-        self._btn_exit.grid(column=4, row=0, padx=3, pady=3, sticky=E)
-
+        self._btn_redraw.grid(column=1, row=0, padx=3, pady=3)
+        self._btn_import.grid(column=2, row=0, padx=3, pady=3)
+        self._chk_first.grid(column=3, row=0, padx=3, pady=3)
         self._lbl_min.grid(column=0, row=1, padx=3, pady=3, sticky=W)
         self._lbl_minlat.grid(column=1, row=1, padx=3, pady=3, sticky=E)
         self._ent_minlat.grid(column=2, row=1, padx=3, pady=3)
@@ -168,62 +157,102 @@ class ImportMapDialog(Toplevel):
         self._ent_maxlat.grid(column=2, row=2, padx=3, pady=3)
         self._lbl_maxlon.grid(column=3, row=2, padx=3, pady=3, sticky=E)
         self._ent_maxlon.grid(column=4, row=2, padx=3, pady=3)
-        self._lbl_status.grid(
-            column=0, row=3, columnspan=5, padx=3, pady=3, sticky=(W, E)
-        )
-
-        self.grid_columnconfigure(0, weight=1)
-        self.grid_rowconfigure(0, weight=3)
-        self.grid_rowconfigure(1, weight=1)
+        self.container.grid_columnconfigure(0, weight=10)
+        self.container.grid_rowconfigure(0, weight=10)
+        self.grid_columnconfigure(0, weight=10)
+        self.grid_rowconfigure(0, weight=10)
+        self._frm_body.grid_columnconfigure(0, weight=10)
+        self._frm_body.grid_rowconfigure(0, weight=10)
+        self._can_mapview.grid_columnconfigure(0, weight=10)
+        self._can_mapview.grid_rowconfigure(0, weight=10)
 
     def _attach_events(self):
         """
         Bind events to window.
         """
 
-        self.bind("<Configure>", self._on_resize)
+        self.container.bind("<Configure>", self._on_resize)
+        for ent in (
+            self._lonmin,
+            self._lonmax,
+            self._latmin,
+            self._latmax,
+        ):
+            ent.trace_update(TRACEMODE_WRITE, self._on_update)
 
     def _reset(self):
         """
         Reset application.
         """
 
-        self._canvas_map.delete(ALL)
-        self._btn_import.config(state=DISABLED)
+        self._first.set(0)
+        self._can_mapview.delete(ALL)
+        # self._btn_import.config(state=DISABLED)
         if not HASRASTERIO:
-            self._show_status(
-                "Warning: rasterio library is not installed - bounds must be entered manually"
+            self.set_status(
+                "Warning: rasterio library is not installed - bounds must be entered manually",
+                INFOCOL,
             )
         else:
-            self._show_status()
+            self.set_status("")
 
-    def on_exit(self, *args, **kwargs):
+    def _on_update(self, var, index, mode):
         """
-        Handle Exit button press.
+        Action on updating bounds.
         """
 
-        self.__app.stop_dialog(DLGTIMPORTMAP)
-        self.destroy()
+        self._valid_entries()
 
-    def get_size(self):
+    def _valid_entries(self) -> bool:
         """
-        Get current frame size.
+        Validate bounds entries.
 
-        :return: window size (width, height)
-        :rtype: tuple
+        :return: True/False
+        :rtype: bool
         """
 
-        return (self.winfo_width(), self.winfo_height())
+        valid = True
+        valid = valid & self._ent_minlat.validate(VALFLOAT, -90, 90)
+        valid = valid & self._ent_maxlat.validate(-90, 90)
+        valid = valid & self._ent_minlon.validate(VALFLOAT, -180, 180)
+        valid = valid & self._ent_maxlon.validate(-180, 180)
+        if valid:
+            self.set_status("", INFOCOL)
+            self._btn_import.config(state=NORMAL)
+        else:
+            self.set_status("Error: invalid entry", ERRCOL)
+            self._btn_import.config(state=DISABLED)
+        return valid
 
-    def _on_resize(self, event):
+    def _on_load(self):
         """
-        Resize frame
-
-        :param event event: resize event
+        Load custom map from file.
         """
 
-        self.width, self.height = self.get_size()
-        self.mheight = self._frm_map.winfo_height()
+        self.set_status("")
+        self._custommap = self._open_mapfile()
+        if self._custommap is not None:
+            bounds = self._get_bounds(self._custommap)
+            self._can_mapview.draw_map(
+                maptype=IMPORT, mappath=self._custommap, bounds=bounds, zoom=None
+            )
+            self._btn_import.config(state=NORMAL)
+        else:
+            self._btn_import.config(state=DISABLED)
+
+    def _on_redraw(self, *args, **kwargs):
+        """
+        Handle redraw button press.
+        """
+
+        self._reset()
+        self._can_mapview.draw_map(
+            maptype=IMPORT,
+            mappath=self._custommap,
+            bounds=self._can_mapview.bounds,
+            marker=self._can_mapview.marker,
+            zoom=None,
+        )
 
     def _open_mapfile(self) -> str:
         """
@@ -235,23 +264,13 @@ class ImportMapDialog(Toplevel):
             (("GeoTiff files", "*.tif"), ("all files", "*.*")),
         )
 
-    def _on_load(self):
-        """
-        Load custom map from file.
-        """
-
-        self._show_status()
-        self._custommap = self._open_mapfile()
-        if self._custommap is not None:
-            self._get_bounds(self._custommap)
-            self._draw_map(self._custommap)
-            self._btn_import.config(state=NORMAL)
-
-    def _get_bounds(self, mappath) -> tuple:
+    def _get_bounds(self, mappath) -> Area:
         """
         Get bounds of custom map image file.
 
         :param str mappath: fully qualified path to map file
+        :return: bounds (extent)
+        :rtype: Area
         """
 
         lonmin = latmin = lonmax = latmax = 0.0
@@ -262,58 +281,40 @@ class ImportMapDialog(Toplevel):
                     ras.crs.to_epsg(), 4326, *ras.bounds
                 )
             except Exception:  # pylint: disable=broad-exception-caught
-                self._show_status(
-                    "Warning: image is not georeferenced - bounds must be entered manually"
+                self.set_status(
+                    "Warning: image is not georeferenced - bounds must be entered manually",
+                    ERRCOL,
                 )
 
         self._lonmin.set(round(lonmin, 8))
         self._latmin.set(round(latmin, 8))
         self._lonmax.set(round(lonmax, 8))
         self._latmax.set(round(latmax, 8))
-
-    def _draw_map(self, mappath="") -> ImageTk.PhotoImage:
-        """
-        Display selected custom map image.
-
-        :param str mappath: fully qualified path to map file
-        """
-
-        if mappath != "":
-            self._mapimg = ImageTk.PhotoImage(
-                Image.open(mappath).resize((self.width, self.mheight)),
-                Image.Resampling.BILINEAR,
-            )
-            self._canvas_map.create_image(0, 0, image=self._mapimg, anchor=NW)
+        return Area(
+            float(self._latmin.get()),
+            float(self._lonmin.get()),
+            float(self._latmax.get()),
+            float(self._lonmax.get()),
+        )
 
     def _on_import(self):
         """
         Validate bounds and import custom file into saved config.
         """
 
-        try:
-            lonmin = float(self._lonmin.get())
-            lonmax = float(self._lonmax.get())
-            latmin = float(self._latmin.get())
-            latmax = float(self._latmax.get())
-        except ValueError:
-            self._show_status("Error: invalid bounds")
+        if not self._valid_entries():
             return
 
+        lonmin = float(self._lonmin.get())
+        lonmax = float(self._lonmax.get())
+        latmin = float(self._latmin.get())
+        latmax = float(self._latmax.get())
+
         if lonmax + 180 <= lonmin + 180 or latmax + 90 <= latmin + 90:
-            self._show_status("Error: minimum must be less than maximum")
+            self.set_status("Error: minimum must be less than maximum", ERRCOL)
         else:
-            usermaps = self.__app.saved_config.get("usermaps_l", [])
-            usermaps.append([self._custommap, [latmin, lonmin, latmax, lonmax]])
-            self.__app.saved_config["usermaps_l"] = usermaps
-            self._show_status("Custom map imported", INFOCOL)
-
-    def _show_status(self, msg: str = "", col: str = ERRCOL):
-        """
-        Show error message in status label
-
-        :param str msg: error message
-        :param str col: text colour
-        """
-
-        self._lbl_status.config(text=msg, fg=col)
-        self.update_idletasks()
+            usermaps = self.__app.configuration.get("usermaps_l")
+            idx = 0 if self._first.get() else len(usermaps) + 1
+            usermaps.insert(idx, [self._custommap, [latmin, lonmin, latmax, lonmax]])
+            self.__app.configuration.set("usermaps_l", usermaps)
+            self.set_status("Custom map imported", INFOCOL)
