@@ -11,12 +11,13 @@ update the various user-selectable widgets.
 
 Created on 30 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
 import logging
+from math import degrees
 from time import time
 
 from pynmeagps import NMEAMessage
@@ -86,11 +87,25 @@ class NMEAHandler:
                 self._process_UBX03(parsed_data)
             elif parsed_data.msgID == "QTMVERNO":  # LG290P hardware version
                 self._process_QTMVERNO(parsed_data)
+            elif parsed_data.msgID == "QTMVER":  # LG290P hardware version
+                self._process_QTMVER(parsed_data)
             elif parsed_data.msgID == "QTMPVT":  # LG290P pos, vel, trk
                 self._process_QTMPVT(parsed_data)
+            elif parsed_data.msgID == "QTMSVINSTATUS":  # LG290P SVIN status
+                self._process_QTMSVINSTATUS(parsed_data)
+            elif parsed_data.msgID in (
+                "QTMDRPVA",
+                "QTMINS",
+                "QTMVEHATT",
+                "QTMTAR",
+                "INVMATTIT",
+                "STMDRPVA",
+            ):
+                self._process_IMU(parsed_data)
+            elif parsed_data.msgID == "FMI":  # Feyman IM19 IMU status
+                self._process_FMI(parsed_data)
             elif parsed_data.msgID[0:3] == "QTM" and hasattr(parsed_data, "status"):
                 self._process_QTMACK(parsed_data)
-
         except ValueError:
             pass
 
@@ -126,7 +141,8 @@ class NMEAHandler:
         self.__app.gnss_status.lat = data.lat
         self.__app.gnss_status.lon = data.lon
         self.__app.gnss_status.alt = data.alt
-        self.__app.gnss_status.sep = data.sep
+        self.__app.gnss_status.hae = data.sep + data.alt
+        self.__app.gnss_status.hdop = data.HDOP
         self.__app.gnss_status.fix = fix2desc("GGA", data.quality)
         self.__app.gnss_status.diff_corr = 0 if data.diffAge == "" else 1
         self.__app.gnss_status.diff_age = data.diffAge
@@ -203,7 +219,7 @@ class NMEAHandler:
         :param pynmeagps.NMEAMessage data: parsed GSV sentence
         """
 
-        show_unused = self.__app.frm_settings.config["unusedsat_b"]
+        show_unused = self.__app.configuration.get("unusedsat_b")
         self.gsv_data = {}
         gsv_dict = {}
         now = time()
@@ -297,8 +313,7 @@ class NMEAHandler:
         """
         # pylint: disable=consider-using-dict-items
 
-        settings = self.__app.frm_settings.config
-        show_unused = settings["unusedsat_b"]
+        show_unused = self.__app.configuration.get("unusedsat_b")
         self.gsv_data = {}
         for i in range(data.numSv):
             idx = f"_{i+1:02d}"
@@ -336,6 +351,20 @@ class NMEAHandler:
         if self.__app.dialog(DLGTNMEA) is not None:
             self.__app.dialog(DLGTNMEA).update_pending(data)
 
+    def _process_QTMVER(self, data: NMEAMessage):
+        """
+        Process QTMVER sentence - Quectel Hardware Version.
+        This gets sent whenever receiver is restarted.
+
+        :param pynmeagps.NMEAMessage data: parsed QTMVER sentence
+        """
+
+        self._process_QTMVERNO(data)
+
+        # notify socket server frame that receiver has restarted...
+        if self.__app.frm_settings.frm_socketserver is not None:
+            self.__app.frm_settings.frm_socketserver.update_pending(data)
+
     def _process_QTMPVT(self, data: NMEAMessage):
         """
         Process QTMPVT sentence - Quectel PVT.
@@ -350,8 +379,8 @@ class NMEAHandler:
         self.__app.gnss_status.track = data.hdg
         self.__app.gnss_status.hdop = data.hdop
         self.__app.gnss_status.pdop = data.pdop
-        self.__app.gnss_status.sep = data.sep
         self.__app.gnss_status.alt = data.alt
+        self.__app.gnss_status.hae = data.sep + data.alt
         self.__app.gnss_status.sip = data.numsv
         self.__app.gnss_status.fix = ["NO FIX", "NO FIX", "2D", "3D"][data.fixtype]
 
@@ -364,3 +393,65 @@ class NMEAHandler:
 
         if self.__app.dialog(DLGTNMEA) is not None:
             self.__app.dialog(DLGTNMEA).update_pending(data)
+
+    def _process_QTMSVINSTATUS(self, data: NMEAMessage):
+        """
+        Process QTMSVINSTATUS sentence - Survey In Status.
+
+        :param NMEAMessage data:QTMSVINSTATUS parsed message
+        """
+
+        valid = 1 if data.valid == 2 else 0
+        active = data.valid == 1
+        if self.__app.frm_settings.frm_socketserver is not None:
+            self.__app.frm_settings.frm_socketserver.svin_countdown(
+                data.obs, valid, active
+            )
+
+    def _process_FMI(self, data: NMEAMessage):
+        """
+        Process GMFMI sentence - Feyman IM19 IMU status.
+
+        Roll, Pitch and Yaw are in radians.
+
+        :param NMEAMessage data: GPFMI message
+        """
+
+        try:  # cater for pynmeagps<1.0.50
+            self.__app.gnss_status.utc = data.time  # datetime.time
+            self.__app.gnss_status.lat = data.lat
+            self.__app.gnss_status.lon = data.lon
+            self.__app.gnss_status.alt = data.alt
+            self.__app.gnss_status.sip = data.numSV
+            self.__app.gnss_status.diff_age = data.diffAge
+            ims = self.__app.gnss_status.imu_data
+            ims["source"] = data.identity
+            ims["roll"] = round(degrees(data.roll), 4)
+            ims["pitch"] = round(degrees(data.pitch), 4)
+            ims["yaw"] = round(degrees(data.yaw), 4)
+            ims["status"] = data.status
+        except (KeyError, AttributeError):
+            pass
+
+    def _process_IMU(self, data: NMEAMessage):
+        """
+        Process IMU status from various proprietary NMEA sentences:
+        PQTMINS, PQTMDRPVA, PQTMVEHATT, PQTMTAR, PINVMATTIT, PSTMDRPVA
+
+        Roll, Pitch and Yaw (Heading) are in degrees.
+
+        :param NMEAMessage data: PQTM* message
+        """
+
+        try:
+            ims = self.__app.gnss_status.imu_data
+            ims["source"] = data.identity
+            ims["roll"] = round(data.roll, 4)
+            ims["pitch"] = round(data.pitch, 4)
+            if hasattr(data, "yaw"):
+                ims["yaw"] = round(data.yaw, 4)
+            elif hasattr(data, "heading"):
+                ims["yaw"] = round(data.heading, 4)
+            ims["status"] = ""
+        except (TypeError, KeyError, AttributeError):
+            pass
