@@ -1,20 +1,18 @@
 """
 nmea_client_dialog.py
 
-NTRIP client container dialog
+NTRIP client configuration dialog.
 
-This is the pop-up dialog containing the various
-NTRIP client configuration functions.
-
-NB: The initial configuration for the NTRIP client
-(pygnssutils.GNSSNTRIPClient) is set in app.update_NTRIP_handler().
+Initial settings are from the saved configuration.
 Once started, the persisted state for the NTRIP client is held in
 the threaded NTRIP handler itself, NOT in this frame.
 
+The dialog may be closed while the NTRIP client is running.
+
 Created on 2 Apr 2022
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -40,12 +38,10 @@ from tkinter import (
     Spinbox,
     StringVar,
     TclError,
-    Toplevel,
     W,
     ttk,
 )
 
-from PIL import Image, ImageTk
 from pygnssutils import NOGGA
 from pygnssutils.helpers import find_mp_distance
 
@@ -54,14 +50,9 @@ from pygpsclient.globals import (
     DISCONNECTED,
     ERRCOL,
     GGA_INTERVALS,
-    ICON_CONN,
-    ICON_DISCONN,
-    ICON_EXIT,
     INFOCOL,
-    POPUP_TRANSIENT,
     READONLY,
-    RPTDELAY,
-    SAVED_CONFIG,
+    TRACEMODE_WRITE,
     UBX_CFGMSG,
     UBX_CFGPRT,
     UBX_CFGRATE,
@@ -71,11 +62,11 @@ from pygpsclient.globals import (
     UBX_PRESET,
     UI,
     UIK,
+    VALFLOAT,
 )
-from pygpsclient.helpers import MAXALT, VALFLOAT, get_mp_info, valid_entry
-from pygpsclient.socketconfig_frame import SocketConfigFrame
+from pygpsclient.helpers import MAXALT, get_mp_info
+from pygpsclient.socketconfig_ntrip_frame import SocketConfigNtripFrame
 from pygpsclient.strings import (
-    DLGNTRIPCONFIG,
     DLGTNTRIP,
     LBLGGAFIXED,
     LBLGGALIVE,
@@ -86,6 +77,7 @@ from pygpsclient.strings import (
     LBLNTRIPUSER,
     LBLNTRIPVERSION,
 )
+from pygpsclient.toplevel_dialog import ToplevelDialog
 
 NTRIP_VERSIONS = ("2.0", "1.0")
 KM2MILES = 0.6213712
@@ -94,9 +86,12 @@ IP6 = "IPv6"
 RTCM = "RTCM"
 SPARTN = "SPARTN"
 NTRIP_SPARTN = "ppntrip.services.u-blox.com"
+TCPIPV4 = "IPv4"
+TCPIPV6 = "IPv6"
+MINDIM = (500, 505)
 
 
-class NTRIPConfigDialog(Toplevel):
+class NTRIPConfigDialog(ToplevelDialog):
     """,
     NTRIPConfigDialog class.
     """
@@ -113,17 +108,8 @@ class NTRIPConfigDialog(Toplevel):
         self.__app = app  # Reference to main application class
         self.logger = getLogger(__name__)
         self.__master = self.__app.appmaster  # Reference to root class (Tk)
-        self._saved_config = kwargs.pop(SAVED_CONFIG, {})
 
-        Toplevel.__init__(self, app)
-        if POPUP_TRANSIENT:
-            self.transient(self.__app)
-        self.resizable(False, False)
-        self.title(DLGNTRIPCONFIG)  # pylint: disable=E1102
-        self.protocol("WM_DELETE_WINDOW", self.on_exit)
-        self._img_exit = ImageTk.PhotoImage(Image.open(ICON_EXIT))
-        self._img_conn = ImageTk.PhotoImage(Image.open(ICON_CONN))
-        self._img_disconn = ImageTk.PhotoImage(Image.open(ICON_DISCONN))
+        super().__init__(app, DLGTNTRIP, MINDIM)
         self._cfg_msg_command = None
         self._pending_confs = {
             UBX_MONVER: (),
@@ -134,7 +120,6 @@ class NTRIPConfigDialog(Toplevel):
             UBX_PRESET: (),
             UBX_CFGRATE: (),
         }
-        self._status = StringVar()
         self._ntrip_datatype = StringVar()
         self._ntrip_https = IntVar()
         self._ntrip_version = StringVar()
@@ -154,8 +139,9 @@ class NTRIPConfigDialog(Toplevel):
 
         self._body()
         self._do_layout()
-        self._attach_events()
         self._reset()
+        self._attach_events()
+        self._finalise()
 
     def _body(self):
         """
@@ -163,38 +149,16 @@ class NTRIPConfigDialog(Toplevel):
         """
         # pylint: disable=unnecessary-lambda
 
-        self._frm_container = Frame(self, borderwidth=2, relief="groove")
-        # use NTRIP config settings in frm_socket
-        saved_ntrip_config = {
-            "sockclienthost_s": self._saved_config.get("ntripclienthost_s", ""),
-            "sockclientport_n": self._saved_config.get("ntripclientport_n", 2101),
-            "sockclientprotocol_s": self._saved_config.get(
-                "ntripclientprotocol_s", "IPv4"
-            ),
-            "sockclientflowinfo_n": self._saved_config.get("ntripclientflowinfo_n", 0),
-            "sockclientscopeid_n": self._saved_config.get("ntripclientscopeid_n", 0),
-            "sockclienthttps_b": self._saved_config.get("ntripclienthttps_b", 0),
-            "protocols_l": [IP4, IP6],
-        }
-        self._frm_socket = SocketConfigFrame(
+        self._frm_body = Frame(self.container, borderwidth=2, relief="groove")
+        self._frm_socket = SocketConfigNtripFrame(
             self.__app,
-            self._frm_container,
-            saved_config=saved_ntrip_config,
+            self._frm_body,
+            protocols=[TCPIPV4, TCPIPV6],
             server_callback=self._on_server,
         )
-        self._frm_status = Frame(self._frm_container, borderwidth=2, relief="groove")
-        self._lbl_status = Label(self._frm_status, textvariable=self._status, anchor=W)
-        self._btn_exit = Button(
-            self._frm_status,
-            image=self._img_exit,
-            width=55,
-            fg=ERRCOL,
-            command=self.on_exit,
-            font=self.__app.font_md,
-        )
-        self._lbl_mountpoint = Label(self._frm_container, text=LBLNTRIPMOUNT)
+        self._lbl_mountpoint = Label(self._frm_body, text=LBLNTRIPMOUNT)
         self._ent_mountpoint = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_mountpoint,
             state=NORMAL,
             relief="sunken",
@@ -202,111 +166,105 @@ class NTRIPConfigDialog(Toplevel):
         )
 
         self._lbl_mpdist = Label(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_mpdist,
             width=30,
             anchor=W,
         )
-        self._lbl_sourcetable = Label(self._frm_container, text=LBLNTRIPSTR)
+        self._lbl_sourcetable = Label(self._frm_body, text=LBLNTRIPSTR)
         self._lbx_sourcetable = Listbox(
-            self._frm_container,
+            self._frm_body,
             height=4,
             relief="sunken",
             width=55,
         )
-        self._scr_sourcetablev = Scrollbar(self._frm_container, orient=VERTICAL)
-        self._scr_sourcetableh = Scrollbar(self._frm_container, orient=HORIZONTAL)
+        self._scr_sourcetablev = Scrollbar(self._frm_body, orient=VERTICAL)
+        self._scr_sourcetableh = Scrollbar(self._frm_body, orient=HORIZONTAL)
         self._lbx_sourcetable.config(yscrollcommand=self._scr_sourcetablev.set)
         self._lbx_sourcetable.config(xscrollcommand=self._scr_sourcetableh.set)
         self._scr_sourcetablev.config(command=self._lbx_sourcetable.yview)
         self._scr_sourcetableh.config(command=self._lbx_sourcetable.xview)
 
-        self._lbl_ntripversion = Label(self._frm_container, text=LBLNTRIPVERSION)
+        self._lbl_ntripversion = Label(self._frm_body, text=LBLNTRIPVERSION)
         self._spn_ntripversion = Spinbox(
-            self._frm_container,
+            self._frm_body,
             values=(NTRIP_VERSIONS),
             width=4,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
             textvariable=self._ntrip_version,
             state=READONLY,
         )
-        self._lbl_datatype = Label(self._frm_container, text="Data Type")
+        self._lbl_datatype = Label(self._frm_body, text="Data Type")
         self._spn_datatype = Spinbox(
-            self._frm_container,
+            self._frm_body,
             values=(RTCM, SPARTN),
             width=8,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
             textvariable=self._ntrip_datatype,
             state=READONLY,
         )
-        self._lbl_user = Label(self._frm_container, text=LBLNTRIPUSER)
+        self._lbl_user = Label(self._frm_body, text=LBLNTRIPUSER)
         self._ent_user = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_user,
             state=NORMAL,
             relief="sunken",
             width=50,
         )
-        self._lbl_password = Label(self._frm_container, text=LBLNTRIPPWD)
+        self._lbl_password = Label(self._frm_body, text=LBLNTRIPPWD)
         self._ent_password = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_password,
             state=NORMAL,
             relief="sunken",
             width=20,
             show="*",
         )
-        self._lbl_ntripggaint = Label(self._frm_container, text=LBLNTRIPGGAINT)
+        self._lbl_ntripggaint = Label(self._frm_body, text=LBLNTRIPGGAINT)
         self._spn_ntripggaint = Spinbox(
-            self._frm_container,
+            self._frm_body,
             values=(GGA_INTERVALS),
             width=5,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
             textvariable=self._ntrip_gga_interval,
             state=READONLY,
         )
         self._rad_ggalive = Radiobutton(
-            self._frm_container, text=LBLGGALIVE, variable=self._ntrip_gga_mode, value=0
+            self._frm_body, text=LBLGGALIVE, variable=self._ntrip_gga_mode, value=0
         )
         self._rad_ggafixed = Radiobutton(
-            self._frm_container,
+            self._frm_body,
             text=LBLGGAFIXED,
             variable=self._ntrip_gga_mode,
             value=1,
         )
-        self._lbl_lat = Label(self._frm_container, text="Ref Latitude")
+        self._lbl_lat = Label(self._frm_body, text="Ref Latitude")
         self._ent_lat = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_gga_lat,
             state=NORMAL,
             relief="sunken",
             width=15,
         )
-        self._lbl_lon = Label(self._frm_container, text="Ref Longitude")
+        self._lbl_lon = Label(self._frm_body, text="Ref Longitude")
         self._ent_lon = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_gga_lon,
             state=NORMAL,
             relief="sunken",
             width=15,
         )
-        self._lbl_alt = Label(self._frm_container, text="Ref Elevation m")
+        self._lbl_alt = Label(self._frm_body, text="Ref Elevation m")
         self._ent_alt = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_gga_alt,
             state=NORMAL,
             relief="sunken",
             width=15,
         )
-        self._lbl_sep = Label(self._frm_container, text="Ref Separation m")
+        self._lbl_sep = Label(self._frm_body, text="Ref Separation m")
         self._ent_sep = Entry(
-            self._frm_container,
+            self._frm_body,
             textvariable=self._ntrip_gga_sep,
             state=NORMAL,
             relief="sunken",
@@ -314,17 +272,17 @@ class NTRIPConfigDialog(Toplevel):
         )
 
         self._btn_connect = Button(
-            self._frm_container,
+            self._frm_body,
             width=45,
             height=35,
-            image=self._img_conn,
+            image=self.img_conn,
             command=lambda: self._connect(),
         )
         self._btn_disconnect = Button(
-            self._frm_container,
+            self._frm_body,
             width=45,
             height=35,
-            image=self._img_disconn,
+            image=self.img_disconn,
             command=lambda: self._disconnect(),
             state=DISABLED,
         )
@@ -335,25 +293,13 @@ class NTRIPConfigDialog(Toplevel):
         """
 
         # top of grid
-        col = 0
-        row = 0
-        self._frm_container.grid(
-            column=col,
-            row=row,
-            columnspan=5,
-            rowspan=22,
-            padx=3,
-            pady=3,
-            ipadx=5,
-            ipady=5,
-            sticky=(N, S, W, E),
-        )
+        self._frm_body.grid(column=0, row=0, sticky=(N, S, E, W))
 
         # body of grid
         self._frm_socket.grid(
             column=0, row=0, columnspan=3, rowspan=3, padx=3, pady=3, sticky=W
         )
-        ttk.Separator(self._frm_container).grid(
+        ttk.Separator(self._frm_body).grid(
             column=0, row=3, columnspan=5, padx=3, pady=3, sticky=(W, E)
         )
         self._lbl_mountpoint.grid(column=0, row=4, padx=3, pady=3, sticky=W)
@@ -375,7 +321,7 @@ class NTRIPConfigDialog(Toplevel):
         self._ent_password.grid(
             column=1, row=13, columnspan=2, padx=3, pady=3, sticky=W
         )
-        ttk.Separator(self._frm_container).grid(
+        ttk.Separator(self._frm_body).grid(
             column=0, row=14, columnspan=5, padx=3, pady=3, sticky=(W, E)
         )
         self._lbl_ntripggaint.grid(column=0, row=15, padx=2, pady=3, sticky=W)
@@ -390,31 +336,12 @@ class NTRIPConfigDialog(Toplevel):
         self._ent_alt.grid(column=1, row=18, columnspan=2, padx=3, pady=2, sticky=W)
         self._lbl_sep.grid(column=2, row=18, padx=3, pady=2, sticky=W)
         self._ent_sep.grid(column=3, row=18, columnspan=2, padx=3, pady=2, sticky=W)
-        ttk.Separator(self._frm_container).grid(
+        ttk.Separator(self._frm_body).grid(
             column=0, row=19, columnspan=5, padx=3, pady=3, sticky=(W, E)
         )
         self._btn_connect.grid(column=0, row=20, padx=3, pady=3, sticky=W)
         self._btn_disconnect.grid(column=1, row=20, padx=3, pady=3, sticky=W)
 
-        # bottom of grid
-        row = 21
-        col = 0
-        (colsp, rowsp) = self._frm_container.grid_size()
-        self._frm_status.grid(column=col, row=row, columnspan=colsp, sticky=(W, E))
-        self._lbl_status.grid(
-            column=0, row=0, columnspan=colsp - 1, ipadx=3, ipady=3, sticky=(W, E)
-        )
-        self._btn_exit.grid(column=colsp - 1, row=0, ipadx=3, ipady=3, sticky=E)
-
-        for frm in (self._frm_container, self._frm_status):
-            for i in range(colsp):
-                frm.grid_columnconfigure(i, weight=1)
-            for i in range(rowsp):
-                frm.grid_rowconfigure(i, weight=1)
-
-        self._frm_container.option_add("*Font", self.__app.font_sm)
-        self._frm_status.option_add("*Font", self.__app.font_sm)
-
     def _attach_events(self):
         """
         Set up event listeners.
@@ -422,6 +349,24 @@ class NTRIPConfigDialog(Toplevel):
 
         self._lbx_sourcetable.bind("<<ListboxSelect>>", self._on_select_mp)
 
+        for setting in (
+            self._ntrip_datatype,
+            self._ntrip_https,
+            self._ntrip_version,
+            self._ntrip_mountpoint,
+            self._ntrip_mpdist,
+            self._ntrip_user,
+            self._ntrip_password,
+            self._ntrip_gga_interval,
+            self._ntrip_gga_mode,
+            self._ntrip_gga_lat,
+            self._ntrip_gga_lon,
+            self._ntrip_gga_alt,
+            self._ntrip_gga_sep,
+        ):
+            setting.trace_add(TRACEMODE_WRITE, self._on_update_config)
+        # self.bind("<Configure>", self._on_resize)
+
     def _reset(self):
         """
         Reset configuration widgets.
@@ -430,6 +375,31 @@ class NTRIPConfigDialog(Toplevel):
         self._get_settings()
         self.set_controls(self._connected)
 
+    def _on_update_config(self, var, index, mode):  # pylint: disable=unused-argument
+        """
+        Update in-memory configuration if setting is changed.
+        """
+
+        try:
+            self.update()
+            cfg = self.__app.configuration
+            cfg.set("ntripclientdatatype_s", self._ntrip_datatype.get())
+            cfg.set("ntripclienthttps_b", int(self._ntrip_https.get()))
+            cfg.set("ntripclientversion_s", self._ntrip_version.get())
+            cfg.set("ntripclientmountpoint_s", self._ntrip_mountpoint.get())
+            cfg.set("ntripclientuser_s", self._ntrip_user.get())
+            cfg.set("ntripclientpassword_s", self._ntrip_password.get())
+            ggaint = self._ntrip_gga_interval.get()
+            ggaint = NOGGA if ggaint == "None" else int(ggaint)
+            cfg.set("ntripclientggainterval_n", ggaint)
+            cfg.set("ntripclientggamode_b", self._ntrip_gga_mode.get())
+            cfg.set("ntripclientreflat_f", float(self._ntrip_gga_lat.get()))
+            cfg.set("ntripclientreflon_f", float(self._ntrip_gga_lon.get()))
+            cfg.set("ntripclientrefalt_f", float(self._ntrip_gga_alt.get()))
+            cfg.set("ntripclientrefsep_f", float(self._ntrip_gga_sep.get()))
+        except (ValueError, TclError):
+            pass
+
     def set_controls(self, connected: bool, msgt: tuple = None):
         """
         Set App RTK connection status and enable or disable controls
@@ -565,37 +535,54 @@ class NTRIPConfigDialog(Toplevel):
         except TclError:
             pass
 
-    def on_exit(self, *args, **kwargs):  # pylint: disable=unused-argument
-        """
-        Handle Exit button press.
-        """
-
-        self.__app.stop_dialog(DLGTNTRIP)
-        self.destroy()
-
-    def get_size(self):
-        """
-        Get current frame size.
-
-        :return: window size (width, height)
-        :rtype: tuple
-        """
-
-        self.__master.update_idletasks()  # Make sure we know about any resizing
-        return (self.winfo_width(), self.winfo_height())
-
     def _get_settings(self):
         """
-        Get settings from the NTRIP handler (pygnssutils.GNSSNTRIPClient).
+        Get settings from saved configuration or from the running instance of
+        the NTRIP handler (pygnssutils.GNSSNTRIPClient).
         """
 
         self._connected = self.__app.ntrip_handler.connected
-        self._settings = self.__app.ntrip_handler.settings
-        ipprot = self._settings.get("ipprot", AF_INET)
+        if self._connected:
+            # get settings from running instance
+            self._settings = self.__app.ntrip_handler.settings
+        else:
+            # get settings from saved configuration
+            cfg = self.__app.configuration
+            self._settings = {}
+            self._settings["server"] = cfg.get("ntripclientserver_s")
+            self._settings["port"] = cfg.get("ntripclientport_n")
+            self._settings["https"] = cfg.get("ntripclienthttps_b")
+            self._settings["selfsign"] = cfg.get("ntripclientselfsign_b")
+            self._settings["ipprot"] = (
+                AF_INET6 if cfg.get("ntripclientprotocol_s") == IP6 else AF_INET
+            )
+            self._settings["flowinfo"] = cfg.get("ntripclientflowinfo_n")
+            self._settings["scopeid"] = cfg.get("ntripclientscopeid_n")
+            self._settings["mountpoint"] = cfg.get("ntripclientmountpoint_s")
+            self._settings["sourcetable"] = []  # this is generated by the NTRIP caster
+            self._settings["version"] = cfg.get("ntripclientversion_s")
+            self._settings["datatype"] = cfg.get("ntripclientdatatype_s")
+            self._settings["ntripuser"] = cfg.get("ntripclientuser_s")
+            self._settings["ntrippassword"] = cfg.get("ntripclientpassword_s")
+            self._settings["ggainterval"] = cfg.get("ntripclientggainterval_n")
+            self._settings["ggamode"] = cfg.get("ntripclientggamode_b")
+            self._settings["reflat"] = cfg.get("ntripclientreflat_f")
+            self._settings["reflon"] = cfg.get("ntripclientreflon_f")
+            self._settings["refalt"] = cfg.get("ntripclientrefalt_f")
+            self._settings["refsep"] = cfg.get("ntripclientrefsep_f")
+            self._settings["spartndecode"] = cfg.get("spartndecode_b")
+            self._settings["spartnkey"] = cfg.get("spartnkey_s")
+            if self._settings["spartnkey"] == "":
+                self._settings["spartndecode"] = 0
+            # if basedate is provided in config file, it must be an integer gnssTimetag
+            self._settings["spartnbasedate"] = cfg.get("spartnbasedate_n")
+
+        ipprot = self._settings.get("ipprot")
         self._frm_socket.protocol.set(IP6 if ipprot == AF_INET6 else IP4)
         self._frm_socket.server.set(self._settings["server"])
         self._frm_socket.port.set(self._settings["port"])
         self._frm_socket.https.set(self._settings["https"])
+        self._frm_socket.selfsign.set(self._settings["selfsign"])
         self._ntrip_mountpoint.set(self._settings["mountpoint"])
         self._ntrip_version.set(self._settings["version"])
         self._ntrip_datatype.set(self._settings["datatype"])
@@ -623,6 +610,7 @@ class NTRIPConfigDialog(Toplevel):
         self._settings["server"] = self._frm_socket.server.get()
         self._settings["port"] = self._frm_socket.port.get()
         self._settings["https"] = self._frm_socket.https.get()
+        self._settings["selfsign"] = self._frm_socket.selfsign.get()
         self._settings["ipprot"] = (
             AF_INET6 if self._frm_socket.protocol.get() == IP6 else AF_INET
         )
@@ -666,6 +654,7 @@ class NTRIPConfigDialog(Toplevel):
                 server=self._settings["server"],
                 port=self._settings["port"],
                 https=self._settings["https"],
+                selfsign=self._settings["selfsign"],
                 flowinfo=self._settings["flowinfo"],
                 scopeid=self._settings["scopeid"],
                 mountpoint=self._settings["mountpoint"],
@@ -705,10 +694,10 @@ class NTRIPConfigDialog(Toplevel):
 
         valid = self._frm_socket.valid_settings()
         if self._settings["ggamode"] == 1:  # fixed reference
-            valid = valid & valid_entry(self._ent_lat, VALFLOAT, -90.0, 90.0)
-            valid = valid & valid_entry(self._ent_lon, VALFLOAT, -180.0, 180.0)
-            valid = valid & valid_entry(self._ent_alt, VALFLOAT, -MAXALT, MAXALT)
-            valid = valid & valid_entry(self._ent_sep, VALFLOAT, -MAXALT, MAXALT)
+            valid = valid & self._ent_lat.validate(VALFLOAT, -90.0, 90.0)
+            valid = valid & self._ent_lon.validate(VALFLOAT, -180.0, 180.0)
+            valid = valid & self._ent_alt.validate(VALFLOAT, -MAXALT, MAXALT)
+            valid = valid & self._ent_sep.validate(VALFLOAT, -MAXALT, MAXALT)
 
         if not valid:
             self.set_status("ERROR - invalid settings", ERRCOL)
@@ -722,13 +711,12 @@ class NTRIPConfigDialog(Toplevel):
         :param float dist: distance to mountpoint km
         """
 
-        settings = self.__app.frm_settings.config
         if name in (None, ""):
             return
         dist_l = "Distance n/a"
         dist_u = "km"
         if isinstance(dist, float):
-            units = settings["units_s"]
+            units = self.__app.configuration.get("units_s")
             if units in (UI, UIK):
                 dist *= KM2MILES
                 dist_u = "miles"
