@@ -5,17 +5,17 @@ Generic socket configuration Frame subclass
 for use in tkinter applications which require a
 socket configuration facility.
 
-Supply initial settings via `saved-config` keyword argument.
-
 Application icons from https://iconmonstr.com/license/.
 
 Created on 27 Apr 2022
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
+# pylint: disable=unused-argument
+
 from tkinter import (
     DISABLED,
     NORMAL,
@@ -34,21 +34,21 @@ from tkinter import (
 from PIL import Image, ImageTk
 
 from pygpsclient.globals import (
-    DEFAULT_PORT,
-    DEFAULT_SERVER,
     DEFAULT_TLS_PORTS,
     ICON_CONTRACT,
     ICON_EXPAND,
-    RPTDELAY,
-    SAVED_CONFIG,
+    READONLY,
+    TRACEMODE_WRITE,
+    VALINT,
+    VALNONBLANK,
+    VALURL,
 )
-from pygpsclient.helpers import MAXPORT, VALINT, VALURL, valid_entry
+from pygpsclient.helpers import MAXPORT
 
 ADVOFF = "\u25bc"
 ADVON = "\u25b2"
 CONNECTED = 1
 DISCONNECTED = 0
-READONLY = "readonly"
 TCPIPV4 = "TCP IPv4"
 TCPIPV6 = "TCP IPv6"
 UDPIPV4 = "UDP IPv4"
@@ -70,8 +70,8 @@ class SocketConfigFrame(Frame):
         :param kwargs: optional kwargs for value ranges, or to pass to Frame parent class
         """
 
-        self._saved_config = kwargs.pop(SAVED_CONFIG, {})
         self._server_callback = kwargs.pop("server_callback", None)
+        self._protocol_range = kwargs.pop("protocols", PROTOCOLS)
         Frame.__init__(self, container, *args, **kwargs)
 
         self.__app = app
@@ -81,15 +81,17 @@ class SocketConfigFrame(Frame):
         self.server = StringVar()
         self.port = IntVar()
         self.https = IntVar()
+        self.selfsign = IntVar()
         self.protocol = StringVar()
-        self._protocol_range = self._saved_config.get("protocols_l", PROTOCOLS)
+
         self._img_expand = ImageTk.PhotoImage(Image.open(ICON_EXPAND))
         self._img_contract = ImageTk.PhotoImage(Image.open(ICON_CONTRACT))
 
         self._body()
         self._do_layout()
         self.reset()
-        self._attach_events()
+        # self._attach_events() # done in reset
+        self._attach_events1()
 
     def _body(self):
         """
@@ -120,13 +122,10 @@ class SocketConfigFrame(Frame):
             width=12,
             state=READONLY,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
         )
-        self._lbl_https = Label(self._frm_basic, text="HTTPS?")
-        self._chk_https = Checkbutton(
-            self._frm_basic,
-            variable=self.https,
+        self._chk_https = Checkbutton(self._frm_basic, variable=self.https, text="TLS")
+        self._chk_selfsign = Checkbutton(
+            self._frm_basic, variable=self.selfsign, text="Self Sign"
         )
 
     def _do_layout(self):
@@ -141,36 +140,95 @@ class SocketConfigFrame(Frame):
         )
         self._lbl_port.grid(column=0, row=1, padx=2, pady=2, sticky=W)
         self.ent_port.grid(column=1, row=1, padx=2, pady=2, sticky=W)
-        self._lbl_protocol.grid(column=3, row=1, padx=2, pady=2, sticky=W)
-        self._spn_protocol.grid(column=4, row=1, padx=2, pady=2, sticky=W)
-        self._lbl_https.grid(column=0, row=2, padx=2, pady=2, sticky=W)
+        self._lbl_protocol.grid(column=2, row=1, padx=2, pady=2, sticky=W)
+        self._spn_protocol.grid(column=3, row=1, padx=2, pady=2, sticky=W)
         self._chk_https.grid(column=1, row=2, padx=2, pady=2, sticky=W)
+        self._chk_selfsign.grid(column=2, row=2, padx=2, pady=2, sticky=W)
 
-    def _attach_events(self):
+    def _attach_events1(self):
         """
-        Bind events to variables.
+        Bind resize event to frame.
         """
 
         self.bind("<Configure>", self._on_resize)
-        self.port.trace_add("write", callback=self._on_port)
-        if self._server_callback is not None:
-            self.server.trace_add("write", callback=self._server_callback)
+
+    def _attach_events(self, add: bool = True):
+        """
+        Add or remove event bindings to/from widgets.
+
+        (trace_update() is a class extension method defined in globals.py)
+
+        :param bool add: add or remove binding
+        """
+
+        tracemode = TRACEMODE_WRITE
+        self.server.trace_update(tracemode, self._on_update_server, add)
+        self.port.trace_update(tracemode, self._on_update_port, add)
+        self.https.trace_update(tracemode, self._on_update_https, add)
+        self.protocol.trace_update(tracemode, self._on_update_protocol, add)
+        self.selfsign.trace_update(tracemode, self._on_update_selfsign, add)
+
+    def _on_update_server(self, var, index, mode):
+        """
+        Server updated.
+        """
+
+        if not self.ent_server.validate(VALNONBLANK):
+            return
+
+        self.__app.configuration.set("sockclienthost_s", self.server.get())
+
+    def _on_update_port(self, var, index, mode):
+        """
+        Port updated.
+        """
+
+        if not self.ent_port.validate(VALINT, 0, MAXPORT):
+            return
+
+        try:
+            if self.port.get() in DEFAULT_TLS_PORTS:
+                self.https.set(1)
+            else:
+                self.https.set(0)
+            self.__app.configuration.set("sockclientport_n", int(self.port.get()))
+        except TclError:
+            pass
+
+    def _on_update_https(self, var, index, mode):
+        """
+        Action on updating TLS flag.
+        """
+
+        self.__app.configuration.set("sockclienthttps_b", int(self.https.get()))
+
+    def _on_update_protocol(self, var, index, mode):
+        """
+        Action on updating TCP/UDP protocol.
+        """
+
+        self.__app.configuration.set("sockclientprotocol_s", self.protocol.get())
+
+    def _on_update_selfsign(self, var, index, mode):
+        """
+        Action on updating self-sign flag.
+        """
+
+        self.__app.configuration.set("sockclientselfsign_b", int(self.selfsign.get()))
 
     def reset(self):
         """
-        Reset settings to defaults (first value in range).
+        Reset settings to saved configuration.
         """
 
-        self.server.set(self._saved_config.get("sockclienthost_s", DEFAULT_SERVER))
-        self.port.set(self._saved_config.get("sockclientport_n", DEFAULT_PORT))
-        self.protocol.set(
-            self._saved_config.get("sockclientprotocol_s", self._protocol_range[0])
-        )
-        self.https.set(
-            self._saved_config.get(
-                "sockclienthttps_b", 1 if self.port.get() in DEFAULT_TLS_PORTS else 0
-            )
-        )
+        self._attach_events(False)
+        cfg = self.__app.configuration
+        self.server.set(cfg.get("sockclienthost_s"))
+        self.port.set(cfg.get("sockclientport_n"))
+        self.https.set(cfg.get("sockclienthttps_b"))
+        self.selfsign.set(cfg.get("sockclientselfsign_b"))
+        self.protocol.set(cfg.get("sockclientprotocol_s"))
+        self._attach_events(True)
 
     def valid_settings(self) -> bool:
         """
@@ -181,8 +239,8 @@ class SocketConfigFrame(Frame):
         """
 
         valid = True
-        valid = valid & valid_entry(self.ent_server, VALURL)
-        valid = valid & valid_entry(self.ent_port, VALINT, 0, MAXPORT)
+        valid = valid & self.ent_server.validate(VALURL)
+        valid = valid & self.ent_port.validate(VALINT, 0, MAXPORT)
         return valid
 
     def set_status(self, status: int = DISCONNECTED):
@@ -200,14 +258,14 @@ class SocketConfigFrame(Frame):
             self._lbl_port,
             self.ent_port,
             self._lbl_protocol,
-            self._lbl_https,
             self._chk_https,
+            self._chk_selfsign,
         ):
             widget.configure(state=(NORMAL if status == DISCONNECTED else DISABLED))
         for widget in (self._spn_protocol,):
             widget.configure(state=(READONLY if status == DISCONNECTED else DISABLED))
 
-    def _on_resize(self, event):  # pylint: disable=unused-argument
+    def _on_resize(self, event):
         """
         Resize frame.
 
@@ -215,18 +273,3 @@ class SocketConfigFrame(Frame):
         """
 
         self.__app.frm_settings.on_expand()
-
-    def _on_port(self, var, index, mode):  # pylint: disable=unused-argument
-        """
-        Callback when port changed.
-
-        :param event event: write event
-        """
-
-        try:
-            if self.port.get() in DEFAULT_TLS_PORTS:
-                self.https.set(1)
-            else:
-                self.https.set(0)
-        except TclError:
-            pass
