@@ -12,12 +12,12 @@ to be increased if the correction source is intermittent.
 
 Created on 26 Jan 2023
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
-from datetime import datetime, timedelta
+from datetime import datetime, timedelta, timezone
 from tkinter import (
     NORMAL,
     Button,
@@ -35,6 +35,7 @@ from tkinter import (
 )
 
 from PIL import Image, ImageTk
+from pyspartn import date2timetag
 from pyubx2 import POLL, SET, SET_LAYER_RAM, TXN_NONE, U1, U2, U4, UBXMessage, val2bytes
 
 from pygpsclient.globals import (
@@ -49,20 +50,19 @@ from pygpsclient.globals import (
     ICON_WARNING,
     OKCOL,
     READONLY,
-    RPTDELAY,
     RXMMSG,
     SPARTN_GNSS,
     SPARTN_KEYLEN,
     SPARTN_SOURCE_IP,
     SPARTN_SOURCE_LB,
-)
-from pygpsclient.helpers import (
+    TRACEMODE_WRITE,
     VALDMY,
     VALHEX,
     VALLEN,
+)
+from pygpsclient.helpers import (
     date2wnotow,
     parse_rxmspartnkey,
-    valid_entry,
 )
 from pygpsclient.spartn_json_config import SpartnJsonConfig
 from pygpsclient.strings import (
@@ -142,6 +142,7 @@ class SPARTNGNSSDialog(Frame):
         self._body()
         self._do_layout()
         self._reset()
+        self._attach_events()
 
     def _body(self):
         """
@@ -204,8 +205,6 @@ class SPARTNGNSSDialog(Frame):
             width=4,
             state=READONLY,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
             textvariable=self._dgnssto,
         )
         self._lbl_loadjson = Label(
@@ -298,6 +297,24 @@ class SPARTNGNSSDialog(Frame):
 
         self._poll_config()
 
+    def _attach_events(self):
+        """
+        Set up event listeners.
+        """
+
+        self._spartn_key1.trace_add(TRACEMODE_WRITE, self._on_update_config)
+
+    def _on_update_config(self, var, index, mode):  # pylint: disable=unused-argument
+        """
+        Update in-memory configuration if setting is changed.
+        """
+
+        self.update()
+        self.__app.configuration.set("spartnkey_s", self._spartn_key1.get())
+        self.__app.configuration.set(
+            "spartnbasedate_n", date2timetag(datetime.now(timezone.utc))
+        )
+
     def _valid_gnss_settings(self) -> bool:
         """
         Validate GNSS receiver settings.
@@ -310,17 +327,17 @@ class SPARTNGNSSDialog(Frame):
         # key must be valid hexadecimal and 16 bytes (32 chars) in length
         valid = (
             valid
-            & valid_entry(self._ent_key1, VALHEX)
-            & valid_entry(self._ent_key1, VALLEN, SPARTN_KEYLEN * 2, SPARTN_KEYLEN * 2)
+            & self._ent_key1.validate(VALHEX)
+            & self._ent_key1.validate(VALLEN, SPARTN_KEYLEN * 2, SPARTN_KEYLEN * 2)
         )
         # from date must be valid date in format YYYYMMDD
-        valid = valid & valid_entry(self._ent_valdate1, VALDMY)
+        valid = valid & self._ent_valdate1.validate(VALDMY)
         valid = (
             valid
-            & valid_entry(self._ent_key2, VALHEX)
-            & valid_entry(self._ent_key2, VALLEN, SPARTN_KEYLEN * 2, SPARTN_KEYLEN * 2)
+            & self._ent_key2.validate(VALHEX)
+            & self._ent_key2.validate(VALLEN, SPARTN_KEYLEN * 2, SPARTN_KEYLEN * 2)
         )
-        valid = valid & valid_entry(self._ent_valdate2, VALDMY)
+        valid = valid & self._ent_valdate2.validate(VALDMY)
 
         if not valid:
             self.__container.set_status("ERROR - invalid settings", ERRCOL)
@@ -441,7 +458,7 @@ class SPARTNGNSSDialog(Frame):
         Send command to GNSS receiver.
         """
 
-        self.__app.gnss_outqueue.put(msg.serialize())
+        self.__app.send_to_device(msg.serialize())
 
     def _set_date(self, val: StringVar, dat: datetime):
         """
@@ -464,7 +481,7 @@ class SPARTNGNSSDialog(Frame):
         """
 
         msg = UBXMessage("RXM", RXMMSG, POLL)
-        self.__app.gnss_outqueue.put(msg.serialize())
+        self.__app.send_to_device(msg.serialize())
         self.__container.set_pending(RXMMSG, SPARTN_GNSS)
 
     def update_status(self, msg: UBXMessage):
@@ -482,7 +499,7 @@ class SPARTNGNSSDialog(Frame):
                 self._spartn_key2.set(keydata[1][0])
                 self._spartn_valdate2.set(keydata[1][1].strftime("%Y%m%d"))
                 # save latest key in configuration settings
-                self.__app.saved_config["spartnkey_s"] = self._spartn_key1.get()
+                self.__app.configuration.set("spartnkey_s", self._spartn_key1.get())
                 col = OKCOL
             else:
                 col = ERRCOL
