@@ -10,8 +10,8 @@ Supply initial settings via `config` keyword argument.
 
 Created on 26 Jan 2023
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -29,13 +29,13 @@ from tkinter import (
     S,
     Spinbox,
     StringVar,
+    TclError,
     W,
     ttk,
 )
 
 from PIL import Image, ImageTk
-from pyubx2 import GET, POLL_LAYER_RAM, SET, SET_LAYER_RAM, TXN_NONE, UBXMessage
-from serial import PARITY_NONE
+from pyubx2 import POLL_LAYER_RAM, SET, SET_LAYER_RAM, TXN_NONE, UBXMessage
 
 from pygpsclient.globals import (
     BPSRATES,
@@ -57,17 +57,17 @@ from pygpsclient.globals import (
     MSGMODES,
     NOPORTS,
     OKCOL,
+    PASSTHRU,
     READONLY,
-    RPTDELAY,
-    SAVED_CONFIG,
     SPARTN_EOF_EVENT,
     SPARTN_ERR_EVENT,
     SPARTN_EVENT,
     SPARTN_LBAND,
     TIMEOUTS,
+    TRACEMODE_WRITE,
+    VALINT,
 )
-from pygpsclient.helpers import VALINT, valid_entry
-from pygpsclient.serialconfig_frame import SerialConfigFrame
+from pygpsclient.serialconfig_lband_frame import SerialConfigLbandFrame
 from pygpsclient.strings import CONFIGBAD, CONFIGOK, DLGSPARTNWARN, LBLSPARTNLB
 
 U2MAX = 2e16 - 1
@@ -79,7 +79,6 @@ RESERVED1 = b"\x00"
 CFGSET = "CFG-VALGET/SET"
 CFGPOLL = "CFG-VALGET"
 INPORTS = ("I2C", "UART1", "UART2", "USB", "SPI")
-PASSTHRU = "Passthrough"
 PMP_DATARATES = {
     "B600": 600,
     "B1200": 1200,
@@ -96,7 +95,7 @@ D9S_FACTORY = {
     "descrm": 1,
     "prescrm": 1,
     "descrminit": 23560,
-    "unqword": 16238547128276412563,
+    "unqword": "16238547128276412563",
 }
 D9S_PP_US = {
     "name": "D9S PointPerfect US",
@@ -108,7 +107,7 @@ D9S_PP_US = {
     "descrm": 1,
     "prescrm": 0,
     "descrminit": 26969,
-    "unqword": 16238547128276412563,
+    "unqword": "16238547128276412563",
 }
 D9S_PP_EU = {
     "name": "D9S PointPerfect EU",
@@ -120,7 +119,7 @@ D9S_PP_EU = {
     "descrm": 1,
     "prescrm": 0,
     "descrminit": 26969,
-    "unqword": 16238547128276412563,
+    "unqword": "16238547128276412563",
 }
 # D9S default configuration
 D9S_CONFIG = D9S_PP_US  # D9S_FACTORY
@@ -146,7 +145,6 @@ class SpartnLbandDialog(Frame):
         self.__app = app  # Reference to main application class
         self.__master = self.__app.appmaster  # Reference to root class (Tk)
         self.__container = container  # container frame
-        self._saved_config = kwargs.pop(SAVED_CONFIG, {})
 
         Frame.__init__(self, self.__container.container, *args, **kwargs)
 
@@ -178,8 +176,8 @@ class SpartnLbandDialog(Frame):
 
         self._body()
         self._do_layout()
-        self._attach_events()
         self._reset()
+        self._attach_events()
 
     def _body(self):
         """
@@ -187,26 +185,15 @@ class SpartnLbandDialog(Frame):
         """
         # pylint: disable=unnecessary-lambda
         self._lbl_corrlband = Label(self, text=LBLSPARTNLB)
+
         # Correction receiver serial port configuration panel
-        saved_lband_config = {
-            "bpsrate_n": self.__app.saved_config.get("lbandclientbpsrate_n", 9600),
-            "databits_n": self.__app.saved_config.get("lbandclientdatabits_n", 8),
-            "stopbits_f": self.__app.saved_config.get("lbandclientstopbits_f", 1.0),
-            "parity_s": self.__app.saved_config.get("lbandclientparity_s", PARITY_NONE),
-            "rtscts_b": self.__app.saved_config.get("lbandclientrtscts_b", 0),
-            "xonxoff_b": self.__app.saved_config.get("lbandclientxonxoff_b", 0),
-            "timeout_f": self.__app.saved_config.get("lbandclienttimeout_f", 0.1),
-            "msgmode_n": self.__app.saved_config.get("lbandclientmsgmode_n", GET),
-            "userport_s": self.__app.saved_config.get("spartnport_s", ""),
-        }
-        self._frm_spartn_serial = SerialConfigFrame(
+        self._frm_spartn_serial = SerialConfigLbandFrame(
             self.__app,
             self,
-            preselect=KNOWNGPS,
+            recognised=KNOWNGPS,
             timeouts=TIMEOUTS,
             bpsrates=BPSRATES,
             msgmodes=list(MSGMODES.keys()),
-            saved_config=saved_lband_config,
         )
         self._lbl_freq = Label(self, text="L-Band Frequency (Hz)")
         self._ent_freq = Entry(
@@ -255,8 +242,6 @@ class SpartnLbandDialog(Frame):
             state=READONLY,
             width=5,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
         )
         self._chk_descrm = Checkbutton(
             self,
@@ -292,8 +277,6 @@ class SpartnLbandDialog(Frame):
             state=READONLY,
             width=10,
             wrap=True,
-            repeatdelay=RPTDELAY,
-            repeatinterval=RPTDELAY,
         )
         self._lbl_ebno = Label(self, text="")
         self._lbl_fec = Label(self, text="")
@@ -367,52 +350,67 @@ class SpartnLbandDialog(Frame):
         """
 
         self.__master.bind(SPARTN_ERR_EVENT, self.on_error)
+        for setting in (
+            self._enabledbg,
+            self._spartn_freq,
+            self._spartn_schwin,
+            self._spartn_usesid,
+            self._spartn_sid,
+            self._spartn_drat,
+            self._spartn_descrm,
+            self._spartn_prescrm,
+            self._spartn_descrminit,
+            self._spartn_unqword,
+            self._spartn_outport,
+        ):
+            setting.trace_add(TRACEMODE_WRITE, self._on_update_config)
 
     def _reset(self):
         """
         Reset configuration widgets.
         """
 
+        cfg = self.__app.configuration
         self._saveconfig.set(0)
-        self._enabledbg.set(self.__app.saved_config.get("lbandclientdebug_b", 0))
-        self._spartn_drat.set(
-            self.__app.saved_config.get("lbandclientdrat_n", PMP_DATARATES["B2400"])
-        )
-        self._spartn_freq.set(
-            self.__app.saved_config.get("lbandclientfreq_n", D9S_CONFIG["freq"])
-        )
-        self._spartn_schwin.set(
-            self.__app.saved_config.get("lbandclientschwin_n", D9S_CONFIG["schwin"])
-        )
-        self._spartn_usesid.set(
-            self.__app.saved_config.get("lbandclientusesid_b", D9S_CONFIG["usesid"])
-        )
-        self._spartn_sid.set(
-            self.__app.saved_config.get("lbandclientsid_n", D9S_CONFIG["sid"])
-        )
-        self._spartn_descrm.set(
-            self.__app.saved_config.get("lbandclientdescrm_b", D9S_CONFIG["descrm"])
-        )
-        self._spartn_prescrm.set(
-            self.__app.saved_config.get("lbandclientprescrm_b", D9S_CONFIG["prescrm"])
-        )
-        self._spartn_descrminit.set(
-            self.__app.saved_config.get(
-                "lbandclientdescrminit_b", D9S_CONFIG["descrminit"]
-            )
-        )
-        self._spartn_unqword.set(
-            self.__app.saved_config.get("lbandclientunqword_s", D9S_CONFIG["unqword"])
-        )
-        self._spartn_outport.set(
-            self.__app.saved_config.get("lbandclientoutport_s", PASSTHRU)
-        )
+        self._enabledbg.set(cfg.get("lbandclientdebug_b"))
+        self._spartn_drat.set(cfg.get("lbandclientdrat_n"))
+        self._spartn_freq.set(cfg.get("lbandclientfreq_n"))
+        self._spartn_schwin.set(cfg.get("lbandclientschwin_n"))
+        self._spartn_usesid.set(cfg.get("lbandclientusesid_b"))
+        self._spartn_sid.set(cfg.get("lbandclientsid_n"))
+        self._spartn_descrm.set(cfg.get("lbandclientdescrm_b"))
+        self._spartn_prescrm.set(cfg.get("lbandclientprescrm_b"))
+        self._spartn_descrminit.set(cfg.get("lbandclientdescrminit_n"))
+        self._spartn_unqword.set(cfg.get("lbandclientunqword_s"))
+        self._spartn_outport.set(cfg.get("lbandclientoutport_s"))
         self.__container.set_status("")
         if self.__app.rtk_conn_status == CONNECTED_SPARTNLB:
             self.set_controls(CONNECTED_SPARTNLB)
         else:
             self.set_controls(DISCONNECTED)
 
+    def _on_update_config(self, var, index, mode):  # pylint: disable=unused-argument
+        """
+        Update in-memory configuration if setting is changed.
+        """
+
+        self.update()
+        cfg = self.__app.configuration
+        try:
+            cfg.set("lbandclientdebug_b", int(self._enabledbg.get()))
+            cfg.set("lbandclientdrat_n", int(self._spartn_drat.get()))
+            cfg.set("lbandclientfreq_n", int(self._spartn_freq.get()))
+            cfg.set("lbandclientschwin_n", int(self._spartn_schwin.get()))
+            cfg.set("lbandclientusesid_b", int(self._spartn_usesid.get()))
+            cfg.set("lbandclientsid_n", int(self._spartn_sid.get()))
+            cfg.set("lbandclientdescrm_b", int(self._spartn_descrm.get()))
+            cfg.set("lbandclientprescrm_b", int(self._spartn_prescrm.get()))
+            cfg.set("lbandclientdescrminit_n", int(self._spartn_descrminit.get()))
+            cfg.set("lbandclientunqword_s", self._spartn_unqword.get())
+            cfg.set("lbandclientoutport_s", self._spartn_outport.get())
+        except (ValueError, TclError):
+            pass
+
     def set_controls(self, status: int):
         """
         Enable or disable widgets depending on connection status.
@@ -456,13 +454,11 @@ class SpartnLbandDialog(Frame):
         """
 
         valid = True
-        valid = valid & valid_entry(
-            self._ent_freq, VALINT, 1e9, 2e9
-        )  # L-band 1GHz - 2GHz
-        valid = valid & valid_entry(self._ent_schwin, VALINT, 0, U2MAX)  # U2
-        valid = valid & valid_entry(self._ent_sid, VALINT, 0, U2MAX)  # U2
-        valid = valid & valid_entry(self._ent_descraminit, VALINT, 0, U2MAX)  # U2
-        valid = valid & valid_entry(self._ent_unqword, VALINT, 0, U8MAX)  # U8
+        valid = valid & self._ent_freq.validate(VALINT, 1e9, 2e9)  # L-band 1GHz - 2GHz
+        valid = valid & self._ent_schwin.validate(VALINT, 0, U2MAX)  # U2
+        valid = valid & self._ent_sid.validate(VALINT, 0, U2MAX)  # U2
+        valid = valid & self._ent_descraminit.validate(VALINT, 0, U2MAX)  # U2
+        valid = valid & self._ent_unqword.validate(VALINT, 0, U8MAX)  # U8
 
         if not valid:
             self.__container.set_status("ERROR - invalid settings", ERRCOL)
@@ -486,6 +482,7 @@ class SpartnLbandDialog(Frame):
             return
 
         conndict = {
+            "protocol": self.__app.protocol_mask,
             "read_event": SPARTN_EVENT,
             "eof_event": SPARTN_EOF_EVENT,
             "error_event": SPARTN_ERR_EVENT,
@@ -493,13 +490,13 @@ class SpartnLbandDialog(Frame):
             "outqueue": self.__app.spartn_outqueue,
             "socket_inqueue": self.__app.socket_inqueue,
             "conntype": CONNECTED,
-            "msgmode": self.__app.frm_settings.frm_serial.msgmode,
+            "msgmode": self.__app.configuration.get("msgmode_n"),
             "serial_settings": self._frm_spartn_serial,
         }
 
         # start serial stream thread
         self.__app.rtk_conn_status = CONNECTED_SPARTNLB
-        self.__app.spartn_stream_handler.start_read_thread(self.__container, conndict)
+        self.__app.spartn_stream_handler.start(self.__container, conndict)
         self.__container.set_status(
             f"Connected to {frm.port}:{frm.port_desc} @ {frm.bpsrate}", OKCOL
         )
@@ -517,7 +514,7 @@ class SpartnLbandDialog(Frame):
         msg += "Disconnected"
         if self.__app.rtk_conn_status == CONNECTED_SPARTNLB:
             if self.__app.spartn_stream_handler is not None:
-                self.__app.spartn_stream_handler.stop_read_thread()
+                self.__app.spartn_stream_handler.stop()
                 self.__app.rtk_conn_status = DISCONNECTED
                 self.__container.set_status(
                     msg,
