@@ -14,16 +14,15 @@ message queue and sends these to the receiver.
 The StreamHandler class is used by two PyGPSClient 'caller' objects:
 
 - SettingsFrame - i/o with the main GNSS receiver.
-- SpartnLbandDialog - i/o with a SPARTN L-Band receiver, when the
-SPARTN Client is active.
+- SpartnLbandDialog - i/o with a SPARTN L-Band receiver when SPARTN Client active.
 
 The caller object can implement a 'set_status()' method to
 display any status messages output by StreamHandler.
 
 Created on 16 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -33,30 +32,37 @@ import ssl
 from datetime import datetime, timedelta
 from queue import Empty
 from threading import Event, Thread
+from time import sleep
 
 from certifi import where as findcacerts
-from pynmeagps import NMEAMessageError, NMEAParseError
-from pyrtcm import RTCMMessageError, RTCMParseError
-from pyubx2 import (
-    ERR_LOG,
+from pygnssutils import (
     NMEA_PROTOCOL,
+    QGC_PROTOCOL,
     RTCM3_PROTOCOL,
+    SBF_PROTOCOL,
     UBX_PROTOCOL,
-    UBXMessageError,
-    UBXParseError,
-    UBXReader,
+    GNSSError,
+    GNSSReader,
+    check_pemfile,
 )
+from pynmeagps import NMEAMessageError, NMEAParseError, NMEAStreamError
+from pyqgc import QGCMessageError, QGCParseError, QGCStreamError
+from pyrtcm import RTCMMessageError, RTCMParseError, RTCMStreamError
+from pysbf2 import SBFMessageError, SBFParseError, SBFStreamError
+from pyubx2 import ERR_LOG, UBXMessageError, UBXParseError, UBXStreamError
 from pyubxutils import UBXSimulator
 from serial import Serial, SerialException, SerialTimeoutException
 
 from pygpsclient.globals import (
+    ASCII,
+    BSR,
     CONNECTED,
     CONNECTED_FILE,
     CONNECTED_SIMULATOR,
     CONNECTED_SOCKET,
     DEFAULT_BUFSIZE,
     ERRCOL,
-    FILEREAD_INTERVAL,
+    TTY_PROTOCOL,
     UBXSIMULATOR,
 )
 
@@ -80,8 +86,9 @@ class StreamHandler:
 
         self._stream_thread = None
         self._stopevent = Event()
+        self._ttyevent = Event()
 
-    def start_read_thread(self, caller: object, settings: dict):
+    def start(self, caller: object, settings: dict):
         """
         Start the stream read thread.
 
@@ -101,7 +108,7 @@ class StreamHandler:
         )
         self._stream_thread.start()
 
-    def stop_read_thread(self):
+    def stop(self):
         """
         Stop serial reader thread.
         """
@@ -129,10 +136,19 @@ class StreamHandler:
         if conntype == CONNECTED:
             if settings["serial_settings"].port == UBXSIMULATOR:
                 conntype = CONNECTED_SIMULATOR
+        ttydelay = (
+            self.__app.configuration.get("ttydelay_b")
+            * self.__app.configuration.get("guiupdateinterval_f")
+            / 2
+        )
 
         try:
             if conntype == CONNECTED:
                 ser = settings["serial_settings"]
+                if settings["protocol"] & TTY_PROTOCOL:
+                    timeout = 3
+                else:
+                    timeout = ser.timeout
                 with Serial(
                     ser.port,
                     ser.bpsrate,
@@ -141,14 +157,22 @@ class StreamHandler:
                     parity=ser.parity,
                     xonxoff=ser.xonxoff,
                     rtscts=ser.rtscts,
-                    timeout=ser.timeout,
+                    timeout=timeout,
                 ) as stream:
-                    self._readloop(
-                        stopevent,
-                        stream,
-                        settings,
-                        inactivity_timeout,
-                    )
+                    if settings["protocol"] & TTY_PROTOCOL:
+                        self._readlooptty(
+                            stopevent,
+                            stream,
+                            settings,
+                            ttydelay,
+                        )
+                    else:
+                        self._readloop(
+                            stopevent,
+                            stream,
+                            settings,
+                            inactivity_timeout,
+                        )
 
             elif conntype == CONNECTED_FILE:
                 in_filepath = settings["in_filepath"]
@@ -165,6 +189,7 @@ class StreamHandler:
                 server = soc.server.get()
                 port = int(soc.port.get())
                 https = int(soc.https.get())
+                selfsign = int(soc.selfsign.get())
                 if soc.protocol.get()[-4:] == "IPv6":
                     afam = socket.AF_INET6
                     conn = socket.getaddrinfo(server, port)[1][4]
@@ -179,6 +204,11 @@ class StreamHandler:
                     if https:
                         context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
                         context.load_verify_locations(findcacerts())
+                        if selfsign:
+                            pem, _ = check_pemfile()
+                            # context.verify_mode = ssl.CERT_NONE
+                            context.load_verify_locations(pem)
+                            context.check_hostname = False
                         stream = context.wrap_socket(stream, server_hostname=server)
                     stream.connect(conn)
                     if socktype == socket.SOCK_DGRAM:
@@ -192,12 +222,20 @@ class StreamHandler:
 
             elif conntype == CONNECTED_SIMULATOR:
                 with UBXSimulator() as stream:
-                    self._readloop(
-                        stopevent,
-                        stream,
-                        settings,
-                        inactivity_timeout,
-                    )
+                    if settings["protocol"] & TTY_PROTOCOL:
+                        self._readlooptty(
+                            stopevent,
+                            stream,
+                            settings,
+                            ttydelay,
+                        )
+                    else:
+                        self._readloop(
+                            stopevent,
+                            stream,
+                            settings,
+                            inactivity_timeout,
+                        )
 
         except EOFError:
             stopevent.set()
@@ -251,9 +289,14 @@ class StreamHandler:
             self.__master.event_generate(settings["read_event"])
 
         conntype = settings["conntype"]
-        ubr = UBXReader(
+
+        ubr = GNSSReader(
             stream,
-            protfilter=NMEA_PROTOCOL | UBX_PROTOCOL | RTCM3_PROTOCOL,
+            protfilter=NMEA_PROTOCOL
+            | UBX_PROTOCOL
+            | SBF_PROTOCOL
+            | QGC_PROTOCOL
+            | RTCM3_PROTOCOL,
             quitonerror=ERR_LOG,
             bufsize=DEFAULT_BUFSIZE,
             msgmode=settings["msgmode"],
@@ -268,7 +311,11 @@ class StreamHandler:
             try:
                 if conntype in (CONNECTED, CONNECTED_SOCKET) or (
                     conntype == CONNECTED_FILE
-                    and datetime.now() > lastread + timedelta(seconds=FILEREAD_INTERVAL)
+                    and datetime.now()
+                    > lastread
+                    + timedelta(
+                        milliseconds=self.__app.configuration.get("filedelay_n")
+                    )
                 ):
                     raw_data, parsed_data = ubr.read()
                     if raw_data is not None:
@@ -300,12 +347,82 @@ class StreamHandler:
             except (
                 UBXMessageError,
                 UBXParseError,
+                UBXStreamError,
                 NMEAMessageError,
                 NMEAParseError,
+                NMEAStreamError,
                 RTCMMessageError,
                 RTCMParseError,
+                RTCMStreamError,
+                SBFMessageError,
+                SBFParseError,
+                SBFStreamError,
+                QGCMessageError,
+                QGCParseError,
+                QGCStreamError,
+                GNSSError,
             ) as err:
-                parsed_data = f"Error parsing data stream {err}"
-                settings["inqueue"].put((raw_data, parsed_data))
-                self.__master.event_generate(settings["read_event"])
+                _errorhandler(err)
+                continue
+
+    def _readlooptty(
+        self,
+        stopevent: Event,
+        stream: object,
+        settings: dict,
+        delay: float = 0.0,
+    ):
+        """
+        THREADED PROCESS
+        TTY (ASCII) Read stream continously until stop event or stream error.
+
+        :param Event stopevent: thread stop event
+        :param object stream: serial data stream
+        :param dict settings: settings dictionary
+        :param float delay: delay between commands (secs)
+        """
+
+        def _errorhandler(err: Exception):
+            """
+            Stream error handler.
+
+            :param Exception err: error
+            """
+
+            parsed_data = f"Error parsing data stream {err}"
+            settings["inqueue"].put((raw_data, parsed_data))
+            self.__master.event_generate(settings["read_event"])
+
+        raw_data = None
+        while not stopevent.is_set():
+
+            try:
+
+                # write any queued output command to serial stream
+                try:
+                    # while not settings["outqueue"].empty():
+                    cmd = settings["outqueue"].get(False)
+                    if cmd is not None:
+                        stream.write(cmd)
+                    settings["outqueue"].task_done()
+                except Empty:
+                    pass
+
+                if delay:
+                    sleep(delay)
+
+                # read ascii input from serial stream
+                raw_data = b""
+                while stream.in_waiting > 0:
+                    raw_data += stream.read(1)
+
+                # place ascii data on input queue
+                if raw_data != b"":
+                    settings["inqueue"].put(
+                        (raw_data, raw_data.decode(ASCII, errors=BSR))
+                    )
+                    self.__master.event_generate(settings["read_event"])
+
+            except (ValueError, SerialException) as err:
+                _errorhandler(err)
                 continue
