@@ -10,8 +10,8 @@ update the various user-selectable widget panels.
 
 Created on 30 Sep 2020
 
-:author: semuadmin
-:copyright: 2020 SEMU Consulting
+:author: semuadmin (Steve Smith)
+:copyright: 2020 semuadmin
 :license: BSD 3-Clause
 """
 
@@ -60,18 +60,24 @@ class UBXHandler:
         # self.logger.debug(f"data received {parsed_data.identity}")
         if parsed_data.identity[0:3] in ("ACK", "CFG"):
             self._process_ACK(parsed_data)
-        elif parsed_data.identity == "MON-VER":
-            self._process_MONVER(parsed_data)
+        elif parsed_data.identity == "ESF-ALG":
+            self._process_ESF_ALG(parsed_data)
+        elif parsed_data.identity == "HNR-ATT":
+            self._process_HNR_ATT(parsed_data)
+        elif parsed_data.identity == "HNR-PVT":
+            self._process_HNR_PVT(parsed_data)
+        elif parsed_data.identity == "NAV-ATT":
+            self._process_NAV_ATT(parsed_data)
+        elif parsed_data.identity in ("NAV-DOP", "NAV2-DOP"):
+            self._process_NAV_DOP(parsed_data)
         elif parsed_data.identity in ("NAV-POSLLH", "NAV-HPPOSLLH"):
             self._process_NAV_POSLLH(parsed_data)
         elif parsed_data.identity in ("NAV-PVT", "NAV2-PVT"):
             self._process_NAV_PVT(parsed_data)
-        elif parsed_data.identity == "NAV-PVAT":
+        elif parsed_data.identity in ("NAV-PVAT", "NAV2-PVAT"):
             self._process_NAV_PVAT(parsed_data)
         elif parsed_data.identity == "NAV-RELPOSNED":
             self._process_NAV_RELPOSNED(parsed_data)
-        elif parsed_data.identity == "NAV-VELNED":
-            self._process_NAV_VELNED(parsed_data)
         elif parsed_data.identity in ("NAV-SAT", "NAV2-SAT"):
             self._process_NAV_SAT(parsed_data)
         elif parsed_data.identity in ("NAV-STATUS", "NAV2-STATUS"):
@@ -82,18 +88,18 @@ class UBXHandler:
             self._process_NAV_SVINFO(parsed_data)
         elif parsed_data.identity == "NAV-SOL":
             self._process_NAV_SOL(parsed_data)
-        elif parsed_data.identity in ("NAV-DOP", "NAV2-DOP"):
-            self._process_NAV_DOP(parsed_data)
-        elif parsed_data.identity == "HNR-PVT":
-            self._process_HNR_PVT(parsed_data)
-        elif parsed_data.identity == "RXM-RTCM":
-            self._process_RXM_RTCM(parsed_data)
+        elif parsed_data.identity == "NAV-VELNED":
+            self._process_NAV_VELNED(parsed_data)
+        elif parsed_data.identity == "MON-COMMS":
+            self._process_MON_COMMS(parsed_data)
         elif parsed_data.identity == "MON-SPAN":
             self._process_MON_SPAN(parsed_data)
         elif parsed_data.identity == "MON-SYS":
             self._process_MON_SYS(parsed_data)
-        elif parsed_data.identity == "MON-COMMS":
-            self._process_MON_COMMS(parsed_data)
+        elif parsed_data.identity == "MON-VER":
+            self._process_MONVER(parsed_data)
+        elif parsed_data.identity == "RXM-RTCM":
+            self._process_RXM_RTCM(parsed_data)
         elif parsed_data.identity == "RXM-PMP":
             self._process_RXM_PMP(parsed_data)
         elif parsed_data.identity == "RXM-SPARTN-KEY":
@@ -115,7 +121,7 @@ class UBXHandler:
 
         # if Spectrumview or Sysmon widgets are active, send ACKSs there
         if msg.identity in ("ACK-ACK", "ACK-NAK"):
-            wdgs = self.__app.widgets
+            wdgs = self.__app.widget_state.state
             for wdg in (WDGSYSMON, WDGSPECTRUM):
                 if wdgs[wdg][VISIBLE]:
                     if msg.clsID == 6 and msg.msgID == 1:  # CFG-MSG
@@ -166,13 +172,11 @@ class UBXHandler:
                 if gnss in exts[i]:
                     gnss_supported = gnss_supported + gnss + " "
 
-        verdata = {}
-        verdata["swversion"] = sw_version
-        verdata["hwversion"] = hw_version
-        verdata["fwversion"] = fw_version
-        verdata["romversion"] = rom_version
-        verdata["gnss"] = gnss_supported
-        self.__app.gnss_status.version_data = verdata
+        self.__app.gnss_status.version_data["swversion"] = sw_version
+        self.__app.gnss_status.version_data["hwversion"] = hw_version
+        self.__app.gnss_status.version_data["fwversion"] = fw_version
+        self.__app.gnss_status.version_data["romversion"] = rom_version
+        self.__app.gnss_status.version_data["gnss"] = gnss_supported
 
         if self.__app.dialog(DLGTUBX) is not None:
             self.__app.dialog(DLGTUBX).update_pending(msg)
@@ -243,7 +247,7 @@ class UBXHandler:
         self.__app.gnss_status.alt = data.hMSL / 1000  # meters
         self.__app.gnss_status.hacc = data.hAcc / 1000  # meters
         self.__app.gnss_status.vacc = data.vAcc / 1000  # meters
-        self.__app.gnss_status.sep = (data.height - data.hMSL) / 1000  # meters
+        self.__app.gnss_status.hae = data.height / 1000  # meters
 
     def _process_NAV_PVT(self, data: UBXMessage):
         """
@@ -262,7 +266,7 @@ class UBXHandler:
         self.__app.gnss_status.sip = data.numSV
         self.__app.gnss_status.speed = data.gSpeed / 1000  # m/s
         self.__app.gnss_status.track = data.headMot
-        self.__app.gnss_status.sep = (data.height - data.hMSL) / 1000  # meters
+        self.__app.gnss_status.hae = data.height / 1000  # meters
         self.__app.gnss_status.fix = fix2desc("NAV-PVT", data.fixType)
         if data.carrSoln > 0:
             self.__app.gnss_status.fix = fix2desc("NAV-PVT", data.carrSoln + 5)
@@ -287,7 +291,15 @@ class UBXHandler:
         self.__app.gnss_status.alt = data.hMSL / 1000  # meters
         self.__app.gnss_status.speed = data.gSpeed / 1000  # m/s
         self.__app.gnss_status.sip = data.numSV
-        self.__app.gnss_status.sep = (data.height - data.hMSL) / 1000  # meters
+        self.__app.gnss_status.hae = data.height / 1000  # meters
+        ims = self.__app.gnss_status.imu_data
+        ims["source"] = data.identity
+        ims["roll"] = data.vehRoll
+        ims["pitch"] = data.vehPitch
+        ims["yaw"] = data.vehHeading
+        ims["status"] = (
+            (data.vehRollValid << 3) + (data.vehPitchValid << 2) + data.vehHeadingValid
+        )
 
     def _process_NAV_VELNED(self, data: UBXMessage):
         """
@@ -311,8 +323,7 @@ class UBXHandler:
         :param UBXMessage data: NAV-SAT parsed message
         """
 
-        settings = self.__app.frm_settings.config
-        show_unused = settings["unusedsat_b"]
+        show_unused = self.__app.configuration.get("unusedsat_b")
         self.gsv_data = {}
         num_siv = int(data.numSvs)
 
@@ -353,7 +364,10 @@ class UBXHandler:
         :param UBXMessage data: NAV-SVIN parsed message
         """
 
-        self.__app.svin_countdown(data.dur, data.valid, data.active)
+        if self.__app.frm_settings.frm_socketserver is not None:
+            self.__app.frm_settings.frm_socketserver.svin_countdown(
+                data.dur, data.valid, data.active
+            )
 
     def _process_NAV_SVINFO(self, data: UBXMessage):
         """
@@ -364,8 +378,7 @@ class UBXHandler:
         :param UBXMessage data: NAV-SVINFO parsed message
         """
 
-        settings = self.__app.frm_settings.config
-        show_unused = settings["unusedsat_b"]
+        show_unused = self.__app.configuration.get("unusedsat_b")
         self.gsv_data = {}
         num_siv = int(data.numCh)
 
@@ -522,3 +535,45 @@ class UBXHandler:
 
         if self.__app.dialog(DLGTSPARTN) is not None:
             self.__app.dialog(DLGTSPARTN).update_pending(data)
+
+    def _process_ESF_ALG(self, data: UBXMessage):
+        """
+        Process ESF-ALG sentence - External Sensor Fusion IMU Alignment.
+
+        :param UBXMessage data: ESF-ALG message
+        """
+
+        ims = self.__app.gnss_status.imu_data
+        ims["source"] = data.identity
+        ims["roll"] = data.roll
+        ims["pitch"] = data.pitch
+        ims["yaw"] = data.yaw
+        ims["status"] = data.status
+
+    def _process_NAV_ATT(self, data: UBXMessage):
+        """
+        Process NAV_ATT sentence - Navigation Attitude.
+
+        :param UBXMessage data: NAV_ATT message
+        """
+
+        ims = self.__app.gnss_status.imu_data
+        ims["source"] = data.identity
+        ims["roll"] = data.roll
+        ims["pitch"] = data.pitch
+        ims["yaw"] = data.heading
+        ims["status"] = ""
+
+    def _process_HNR_ATT(self, data: UBXMessage):
+        """
+        Process HNR_ATT sentence - High Rate Navigation Attitude..
+
+        :param UBXMessage data: HNR_ATT message
+        """
+
+        ims = self.__app.gnss_status.imu_data
+        ims["source"] = data.identity
+        ims["roll"] = data.roll
+        ims["pitch"] = data.pitch
+        ims["yaw"] = data.heading
+        ims["status"] = ""
